"""
Google Analytics template tags and filters, using the new analytics.js library.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import (
    disable_html,
    get_required_setting,
    is_internal_ip,
)

PROPERTY_ID_RE = re.compile(r'^UA-\d+-\d+$')
SETUP_CODE = """
<script async src="https://www.googletagmanager.com/gtag/js?id={property_id}"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){{dataLayer.push(arguments);}}
  gtag('js', new Date());

  gtag('config', '{property_id}');
</script>
"""

register = Library()


@register.tag
def google_analytics_gtag(parser, token):
    """
    Google Analytics tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your website property ID (as a string) in the
    ``GOOGLE_ANALYTICS_GTAG_PROPERTY_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return GoogleAnalyticsGTagNode()


class GoogleAnalyticsGTagNode(Node):
    def __init__(self):
        self.property_id = get_required_setting(
            'GOOGLE_ANALYTICS_GTAG_PROPERTY_ID', PROPERTY_ID_RE,
            "must be a string looking like 'UA-XXXXXX-Y'")

    def render(self, context):
        html = SETUP_CODE.format(
            property_id=self.property_id,
        )
        if is_internal_ip(context, 'GOOGLE_ANALYTICS'):
            html = disable_html(html, 'Google Analytics')
        return html


def contribute_to_analytical(add_node):
    GoogleAnalyticsGTagNode()  # ensure properly configured
    add_node('head_top', GoogleAnalyticsGTagNode)



"""
Utility function for django-analytical.
"""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


HTML_COMMENT = "<!-- %(service)s disabled on internal IP " \
               "address\n%(html)s\n-->"


def get_required_setting(setting, value_re, invalid_msg):
    """
    Return a constant from ``django.conf.settings``.  The `setting`
    argument is the constant name, the `value_re` argument is a regular
    expression used to validate the setting value and the `invalid_msg`
    argument is used as exception message if the value is not valid.
    """
    try:
        value = getattr(settings, setting)
    except AttributeError:
        raise AnalyticalException("%s setting: not found" % setting)
    if not value:
        raise AnalyticalException("%s setting is not set" % setting)
    value = str(value)
    if not value_re.search(value):
        raise AnalyticalException("%s setting: %s: '%s'"
                                  % (setting, invalid_msg, value))
    return value


def get_user_from_context(context):
    """
    Get the user instance from the template context, if possible.

    If the context does not contain a `request` or `user` attribute,
    `None` is returned.
    """
    try:
        return context['user']
    except KeyError:
        pass
    try:
        request = context['request']
        return request.user
    except (KeyError, AttributeError):
        pass
    return None


def get_user_is_authenticated(user):
    """Check if the user is authenticated.

    This is a compatibility function needed to support both Django 1.x and 2.x;
    Django 2.x turns the function into a proper boolean so function calls will
    fail.
    """
    if callable(user.is_authenticated):
        return user.is_authenticated()
    else:
        return user.is_authenticated


def get_identity(context, prefix=None, identity_func=None, user=None):
    """
    Get the identity of a logged in user from a template context.

    The `prefix` argument is used to provide different identities to
    different analytics services.  The `identity_func` argument is a
    function that returns the identity of the user; by default the
    identity is the username.
    """
    if prefix is not None:
        try:
            return context['%s_identity' % prefix]
        except KeyError:
            pass
    try:
        return context['analytical_identity']
    except KeyError:
        pass
    if getattr(settings, 'ANALYTICAL_AUTO_IDENTIFY', True):
        try:
            if user is None:
                user = get_user_from_context(context)
            if get_user_is_authenticated(user):
                if identity_func is not None:
                    return identity_func(user)
                else:
                    return user.get_username()
        except (KeyError, AttributeError):
            pass
    return None


def get_domain(context, prefix):
    """
    Return the domain used for the tracking code.  Each service may be
    configured with its own domain (called `<name>_domain`), or a
    django-analytical-wide domain may be set (using `analytical_domain`.

    If no explicit domain is found in either the context or the
    settings, try to get the domain from the contrib sites framework.
    """
    domain = context.get('%s_domain' % prefix)
    if domain is None:
        domain = context.get('analytical_domain')
    if domain is None:
        domain = getattr(settings, '%s_DOMAIN' % prefix.upper(), None)
    if domain is None:
        domain = getattr(settings, 'ANALYTICAL_DOMAIN', None)
    if domain is None:
        if 'django.contrib.sites' in settings.INSTALLED_APPS:
            from django.contrib.sites.models import Site
            try:
                domain = Site.objects.get_current().domain
            except (ImproperlyConfigured, Site.DoesNotExist):
                pass
    return domain


def is_internal_ip(context, prefix=None):
    """
    Return whether the visitor is coming from an internal IP address,
    based on information from the template context.

    The prefix is used to allow different analytics services to have
    different notions of internal addresses.
    """
    try:
        request = context['request']
        remote_ip = request.META.get('HTTP_X_FORWARDED_FOR', '')
        if not remote_ip:
            remote_ip = request.META.get('REMOTE_ADDR', '')
        if not remote_ip:
            return False

        internal_ips = None
        if prefix is not None:
            internal_ips = getattr(settings, '%s_INTERNAL_IPS' % prefix, None)
        if internal_ips is None:
            internal_ips = getattr(settings, 'ANALYTICAL_INTERNAL_IPS', None)
        if internal_ips is None:
            internal_ips = getattr(settings, 'INTERNAL_IPS', None)

        return remote_ip in (internal_ips or [])
    except (KeyError, AttributeError):
        return False


def disable_html(html, service):
    """
    Disable HTML code by commenting it out.

    The `service` argument is used to display a friendly message.
    """
    return HTML_COMMENT % {'html': html, 'service': service}


class AnalyticalException(Exception):
    """
    Raised when an exception occurs in any django-analytical code that should
    be silenced in templates.
    """
    silent_variable_failure = True



"""
Tests for the Spring Metrics template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.spring_metrics import SpringMetricsNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(SPRING_METRICS_TRACKING_ID='12345678')
class SpringMetricsTagTestCase(TagTestCase):
    """
    Tests for the ``spring_metrics`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('spring_metrics', 'spring_metrics')
        self.assertTrue("_springMetq.push(['id', '12345678']);" in r, r)

    def test_node(self):
        r = SpringMetricsNode().render(Context({}))
        self.assertTrue("_springMetq.push(['id', '12345678']);" in r, r)

    @override_settings(SPRING_METRICS_TRACKING_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, SpringMetricsNode)

    @override_settings(SPRING_METRICS_TRACKING_ID='123xyz')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, SpringMetricsNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = SpringMetricsNode().render(Context({
            'user': User(email='test@test.com'),
        }))
        self.assertTrue("_springMetq.push(['setdata', {'email': 'test@test.com'}]);" in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = SpringMetricsNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse("_springMetq.push(['setdata', {'email':" in r, r)

    def test_custom(self):
        r = SpringMetricsNode().render(Context({
            'spring_metrics_var1': 'val1',
            'spring_metrics_var2': 'val2',
        }))
        self.assertTrue("_springMetq.push(['setdata', {'var1': 'val1'}]);" in r, r)
        self.assertTrue("_springMetq.push(['setdata', {'var2': 'val2'}]);" in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = SpringMetricsNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Spring Metrics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



# -*- coding: utf-8 -*-
#
# This file is execfile()d with the current directory set to its containing
# directory.

import os
import sys

sys.path.append(os.path.join(os.path.abspath('.'), '_ext'))
sys.path.append(os.path.dirname(os.path.abspath('.')))

import analytical  # noqa


# -- General configuration --------------------------------------------------

project = u'django-analytical'
copyright = u'2011-2016, Joost Cassee <joost@cassee.net>'

release = analytical.__version__
# The short X.Y version.
version = release.rsplit('.', 1)[0]

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx', 'local']
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

add_function_parentheses = True
pygments_style = 'sphinx'

intersphinx_mapping = {
    'http://docs.python.org/2.7': None,
    'http://docs.djangoproject.com/en/1.9':
        'http://docs.djangoproject.com/en/1.9/_objects/',
}


# -- Options for HTML output ------------------------------------------------

html_theme = 'default'
htmlhelp_basename = 'analyticaldoc'


# -- Options for LaTeX output -----------------------------------------------

latex_documents = [
    ('index', 'django-analytical.tex', u'Documentation for django-analytical',
     u'Joost Cassee', 'manual'),
]



"""
Tests for the SnapEngage template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.template import Context
from django.test.utils import override_settings
from django.utils import translation

from analytical.templatetags.snapengage import SnapEngageNode, \
        BUTTON_STYLE_LIVE, BUTTON_STYLE_DEFAULT, BUTTON_STYLE_NONE, \
        BUTTON_LOCATION_LEFT, BUTTON_LOCATION_RIGHT, BUTTON_LOCATION_TOP, \
        BUTTON_LOCATION_BOTTOM, FORM_POSITION_TOP_LEFT
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


WIDGET_ID = 'ec329c69-0bf0-4db8-9b77-3f8150fb977e'


@override_settings(
    SNAPENGAGE_WIDGET_ID=WIDGET_ID,
    SNAPENGAGE_BUTTON=BUTTON_STYLE_DEFAULT,
    SNAPENGAGE_BUTTON_LOCATION=BUTTON_LOCATION_LEFT,
    SNAPENGAGE_BUTTON_OFFSET="55%",
)
class SnapEngageTestCase(TagTestCase):
    """
    Tests for the ``snapengage`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('snapengage', 'snapengage')
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
            '"55%");' in r, r)

    def test_node(self):
        r = SnapEngageNode().render(Context())
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
            '"55%");' in r, r)

    @override_settings(SNAPENGAGE_WIDGET_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, SnapEngageNode)

    @override_settings(SNAPENGAGE_WIDGET_ID='abc')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, SnapEngageNode)

    def test_no_button(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button': BUTTON_STYLE_NONE,
        }))
        self.assertTrue('SnapABug.init("ec329c69-0bf0-4db8-9b77-3f8150fb977e")' in r, r)
        with override_settings(SNAPENGAGE_BUTTON=BUTTON_STYLE_NONE):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.init("ec329c69-0bf0-4db8-9b77-3f8150fb977e")' in r, r)

    def test_live_button(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button': BUTTON_STYLE_LIVE,
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
            '"55%",true);' in r, r)
        with override_settings(SNAPENGAGE_BUTTON=BUTTON_STYLE_LIVE):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
                '"55%",true);' in r, r)

    def test_custom_button(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button': "http://www.example.com/button.png",
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
            '"55%");' in r, r)
        self.assertTrue(
            'SnapABug.setButton("http://www.example.com/button.png");' in r, r)
        with override_settings(
                SNAPENGAGE_BUTTON="http://www.example.com/button.png"):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
                '"55%");' in r, r)
            self.assertTrue(
                'SnapABug.setButton("http://www.example.com/button.png");' in r,
                r)

    def test_button_location_right(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button_location': BUTTON_LOCATION_RIGHT,
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","1",'
            '"55%");' in r, r)
        with override_settings(SNAPENGAGE_BUTTON_LOCATION=BUTTON_LOCATION_RIGHT):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","1",'
                '"55%");' in r, r)

    def test_button_location_top(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button_location': BUTTON_LOCATION_TOP,
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","2",'
            '"55%");' in r, r)
        with override_settings(SNAPENGAGE_BUTTON_LOCATION=BUTTON_LOCATION_TOP):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","2",'
                '"55%");' in r, r)

    def test_button_location_bottom(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button_location': BUTTON_LOCATION_BOTTOM,
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","3",'
            '"55%");' in r, r)
        with override_settings(
                SNAPENGAGE_BUTTON_LOCATION=BUTTON_LOCATION_BOTTOM):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","3",'
                '"55%");' in r, r)

    def test_button_offset(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button_location_offset': "30%",
        }))
        self.assertTrue(
            'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
            '"30%");' in r, r)
        with override_settings(SNAPENGAGE_BUTTON_LOCATION_OFFSET="30%"):
            r = SnapEngageNode().render(Context())
            self.assertTrue(
                'SnapABug.addButton("ec329c69-0bf0-4db8-9b77-3f8150fb977e","0",'
                '"30%");' in r, r)

    def test_button_effect(self):
        r = SnapEngageNode().render(Context({
            'snapengage_button_effect': "-4px",
        }))
        self.assertTrue('SnapABug.setButtonEffect("-4px");' in r, r)
        with override_settings(SNAPENGAGE_BUTTON_EFFECT="-4px"):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setButtonEffect("-4px");' in r, r)

    def test_form_position(self):
        r = SnapEngageNode().render(Context({
            'snapengage_form_position': FORM_POSITION_TOP_LEFT,
        }))
        self.assertTrue('SnapABug.setChatFormPosition("tl");' in r, r)
        with override_settings(SNAPENGAGE_FORM_POSITION=FORM_POSITION_TOP_LEFT):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setChatFormPosition("tl");' in r, r)

    def test_form_top_position(self):
        r = SnapEngageNode().render(Context({
            'snapengage_form_top_position': 40,
        }))
        self.assertTrue('SnapABug.setFormTopPosition(40);' in r, r)
        with override_settings(SNAPENGAGE_FORM_TOP_POSITION=40):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setFormTopPosition(40);' in r, r)

    def test_domain(self):
        r = SnapEngageNode().render(Context({
            'snapengage_domain': "example.com"}))
        self.assertTrue('SnapABug.setDomain("example.com");' in r, r)
        with override_settings(SNAPENGAGE_DOMAIN="example.com"):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setDomain("example.com");' in r, r)

    def test_secure_connection(self):
        r = SnapEngageNode().render(Context({
            'snapengage_secure_connection': True}))
        self.assertTrue('SnapABug.setSecureConnexion();' in r, r)
        with override_settings(SNAPENGAGE_SECURE_CONNECTION=True):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setSecureConnexion();' in r, r)

    def test_show_offline(self):
        r = SnapEngageNode().render(Context({
            'snapengage_show_offline': False,
        }))
        self.assertTrue('SnapABug.allowOffline(false);' in r, r)
        with override_settings(SNAPENGAGE_SHOW_OFFLINE=False):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.allowOffline(false);' in r, r)

    def test_proactive_chat(self):
        r = SnapEngageNode().render(Context({
            'snapengage_proactive_chat': False}))
        self.assertTrue('SnapABug.allowProactiveChat(false);' in r, r)

    def test_screenshot(self):
        r = SnapEngageNode().render(Context({
            'snapengage_screenshots': False,
        }))
        self.assertTrue('SnapABug.allowScreenshot(false);' in r, r)
        with override_settings(SNAPENGAGE_SCREENSHOTS=False):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.allowScreenshot(false);' in r, r)

    def test_offline_screenshots(self):
        r = SnapEngageNode().render(Context({
            'snapengage_offline_screenshots': False,
        }))
        self.assertTrue('SnapABug.showScreenshotOption(false);' in r, r)
        with override_settings(SNAPENGAGE_OFFLINE_SCREENSHOTS=False):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.showScreenshotOption(false);' in r, r)

    def test_sounds(self):
        r = SnapEngageNode().render(Context({'snapengage_sounds': False}))
        self.assertTrue('SnapABug.allowChatSound(false);' in r, r)
        with override_settings(SNAPENGAGE_SOUNDS=False):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.allowChatSound(false);' in r, r)

    @override_settings(SNAPENGAGE_READONLY_EMAIL=False)
    def test_email(self):
        r = SnapEngageNode().render(Context({
            'snapengage_email': 'test@example.com',
        }))
        self.assertTrue('SnapABug.setUserEmail("test@example.com");' in r, r)

    def test_email_readonly(self):
        r = SnapEngageNode().render(Context({
            'snapengage_email': 'test@example.com',
            'snapengage_readonly_email': True,
        }))
        self.assertTrue('SnapABug.setUserEmail("test@example.com",true);' in r, r)
        with override_settings(SNAPENGAGE_READONLY_EMAIL=True):
            r = SnapEngageNode().render(Context({
                'snapengage_email': 'test@example.com',
            }))
            self.assertTrue('SnapABug.setUserEmail("test@example.com",true);' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = SnapEngageNode().render(Context({
            'user': User(username='test', email='test@example.com'),
        }))
        self.assertTrue('SnapABug.setUserEmail("test@example.com");' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = SnapEngageNode().render(Context({
            'user': AnonymousUser(),
        }))
        self.assertFalse('SnapABug.setUserEmail(' in r, r)

    def test_language(self):
        r = SnapEngageNode().render(Context({'snapengage_locale': 'fr'}))
        self.assertTrue('SnapABug.setLocale("fr");' in r, r)
        with override_settings(SNAPENGAGE_LOCALE='fr'):
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setLocale("fr");' in r, r)

    def test_automatic_language(self):
        real_get_language = translation.get_language
        try:
            translation.get_language = lambda: 'fr-ca'
            r = SnapEngageNode().render(Context())
            self.assertTrue('SnapABug.setLocale("fr_CA");' in r, r)
        finally:
            translation.get_language = real_get_language



"""
Tests for the Hotjar template tags.
"""
from django.http import HttpRequest
from django.template import Context, Template, TemplateSyntaxError
from django.test import override_settings

from analytical.templatetags.analytical import _load_template_nodes
from analytical.templatetags.hotjar import HotjarNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


expected_html = """\
<script>
    (function(h,o,t,j,a,r){
        h.hj=h.hj||function(){(h.hj.q=h.hj.q||[]).push(arguments)};
        h._hjSettings={hjid:123456789,hjsv:6};
        a=o.getElementsByTagName('head')[0];
        r=o.createElement('script');r.async=1;
        r.src=t+h._hjSettings.hjid+j+h._hjSettings.hjsv;
        a.appendChild(r);
    })(window,document,'https://static.hotjar.com/c/hotjar-','.js?sv=');
</script>
"""


@override_settings(HOTJAR_SITE_ID='123456789')
class HotjarTagTestCase(TagTestCase):

    maxDiff = None

    def test_tag(self):
        html = self.render_tag('hotjar', 'hotjar')
        self.assertEqual(expected_html, html)

    def test_node(self):
        html = HotjarNode().render(Context({}))
        self.assertEqual(expected_html, html)

    def test_tags_take_no_args(self):
        self.assertRaisesRegexp(
            TemplateSyntaxError,
            r"^'hotjar' takes no arguments$",
            lambda: (Template('{% load hotjar %}{% hotjar "arg" %}')
                     .render(Context({}))),
        )

    @override_settings(HOTJAR_SITE_ID=None)
    def test_no_id(self):
        expected_pattern = r'^HOTJAR_SITE_ID setting is not set$'
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, HotjarNode)

    @override_settings(HOTJAR_SITE_ID='invalid')
    def test_invalid_id(self):
        expected_pattern = (
            r"^HOTJAR_SITE_ID setting: must be \(a string containing\) a number: 'invalid'$")
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, HotjarNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': request})

        actual_html = HotjarNode().render(context)
        disabled_html = '\n'.join([
                '<!-- Hotjar disabled on internal IP address',
                expected_html,
                '-->',
            ])
        self.assertEqual(disabled_html, actual_html)

    def test_contribute_to_analytical(self):
        """
        `hotjar.contribute_to_analytical` registers the head and body nodes.
        """
        template_nodes = _load_template_nodes()
        self.assertEqual({
            'head_top': [],
            'head_bottom': [HotjarNode],
            'body_top': [],
            'body_bottom': [],
        }, template_nodes)



"""
GoSquared template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_identity, \
        is_internal_ip, disable_html, get_required_setting


TOKEN_RE = re.compile(r'^\S+-\S+-\S+$')
TRACKING_CODE = """
    <script type="text/javascript">
      var GoSquared={};
      %(config)s
      (function(w){
        function gs(){
          w._gstc_lt=+(new Date); var d=document;
          var g = d.createElement("script"); g.type = "text/javascript"; g.async = true; g.src = "//d1l6p2sc9645hc.cloudfront.net/tracker.js";
          var s = d.getElementsByTagName("script")[0]; s.parentNode.insertBefore(g, s);
        }
        w.addEventListener?w.addEventListener("load",gs,false):w.attachEvent("onload",gs);
      })(window);
    </script>
"""  # noqa
TOKEN_CODE = 'GoSquared.acct = "%s";'
IDENTIFY_CODE = 'GoSquared.UserName = "%s";'


register = Library()


@register.tag
def gosquared(parser, token):
    """
    GoSquared tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your GoSquared site token in the ``GOSQUARED_SITE_TOKEN`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return GoSquaredNode()


class GoSquaredNode(Node):
    def __init__(self):
        self.site_token = get_required_setting(
                'GOSQUARED_SITE_TOKEN', TOKEN_RE,
                "must be a string looking like XXX-XXXXXX-X")

    def render(self, context):
        configs = [TOKEN_CODE % self.site_token]
        identity = get_identity(context, 'gosquared', self._identify)
        if identity:
            configs.append(IDENTIFY_CODE % identity)
        html = TRACKING_CODE % {
            'token': self.site_token,
            'config': ' '.join(configs),
        }
        if is_internal_ip(context, 'GOSQUARED'):
            html = disable_html(html, 'GoSquared')
        return html

    def _identify(self, user):
        name = user.get_full_name()
        if not name:
            name = user.username
        return name


def contribute_to_analytical(add_node):
    GoSquaredNode()  # ensure properly configured
    add_node('body_bottom', GoSquaredNode)



"""
Google Analytics template tags and filters.

DEPRECATED
"""

from __future__ import absolute_import

import decimal
import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import (
    AnalyticalException,
    disable_html,
    get_domain,
    get_required_setting,
    is_internal_ip,
)

TRACK_SINGLE_DOMAIN = 1
TRACK_MULTIPLE_SUBDOMAINS = 2
TRACK_MULTIPLE_DOMAINS = 3

SCOPE_VISITOR = 1
SCOPE_SESSION = 2
SCOPE_PAGE = 3

PROPERTY_ID_RE = re.compile(r'^UA-\d+-\d+$')
SETUP_CODE = """
    <script type="text/javascript">

      var _gaq = _gaq || [];
      _gaq.push(['_setAccount', '%(property_id)s']);
      %(commands)s
      (function() {
        var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
        ga.src = ('https:' == document.location.protocol ? %(source_scheme)s) + %(source_url)s;
        var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
      })();

    </script>
"""
DOMAIN_CODE = "_gaq.push(['_setDomainName', '%s']);"
NO_ALLOW_HASH_CODE = "_gaq.push(['_setAllowHash', false]);"
TRACK_PAGE_VIEW = "_gaq.push(['_trackPageview']);"
ALLOW_LINKER_CODE = "_gaq.push(['_setAllowLinker', true]);"
CUSTOM_VAR_CODE = "_gaq.push(['_setCustomVar', %(index)s, '%(name)s', " \
                  "'%(value)s', %(scope)s]);"
SITE_SPEED_CODE = "_gaq.push(['_trackPageLoadTime']);"
ANONYMIZE_IP_CODE = "_gaq.push(['_gat._anonymizeIp']);"
SAMPLE_RATE_CODE = "_gaq.push(['_setSampleRate', '%s']);"
SITE_SPEED_SAMPLE_RATE_CODE = "_gaq.push(['_setSiteSpeedSampleRate', '%s']);"
SESSION_COOKIE_TIMEOUT_CODE = "_gaq.push(['_setSessionCookieTimeout', '%s']);"
VISITOR_COOKIE_TIMEOUT_CODE = "_gaq.push(['_setVisitorCookieTimeout', '%s']);"
DEFAULT_SOURCE = ("'https://ssl' : 'http://www'", "'.google-analytics.com/ga.js'")
DISPLAY_ADVERTISING_SOURCE = ("'https://' : 'http://'", "'stats.g.doubleclick.net/dc.js'")

ZEROPLACES = decimal.Decimal('0')
TWOPLACES = decimal.Decimal('0.01')

register = Library()


@register.tag
def google_analytics(parser, token):
    """
    Google Analytics tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your website property ID (as a string) in the
    ``GOOGLE_ANALYTICS_PROPERTY_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return GoogleAnalyticsNode()


class GoogleAnalyticsNode(Node):
    def __init__(self):
        self.property_id = get_required_setting(
            'GOOGLE_ANALYTICS_PROPERTY_ID', PROPERTY_ID_RE,
            "must be a string looking like 'UA-XXXXXX-Y'")

    def render(self, context):
        commands = self._get_domain_commands(context)
        commands.extend(self._get_custom_var_commands(context))
        commands.extend(self._get_other_commands(context))
        commands.append(TRACK_PAGE_VIEW)
        if getattr(settings, 'GOOGLE_ANALYTICS_DISPLAY_ADVERTISING', False):
            source = DISPLAY_ADVERTISING_SOURCE
        else:
            source = DEFAULT_SOURCE
        html = SETUP_CODE % {
            'property_id': self.property_id,
            'commands': " ".join(commands),
            'source_scheme': source[0],
            'source_url': source[1],
        }
        if is_internal_ip(context, 'GOOGLE_ANALYTICS'):
            html = disable_html(html, 'Google Analytics')
        return html

    def _get_domain_commands(self, context):
        commands = []
        tracking_type = getattr(settings, 'GOOGLE_ANALYTICS_TRACKING_STYLE',
                                TRACK_SINGLE_DOMAIN)
        if tracking_type == TRACK_SINGLE_DOMAIN:
            pass
        else:
            domain = get_domain(context, 'google_analytics')
            if domain is None:
                raise AnalyticalException(
                    "tracking multiple domains with Google Analytics requires a domain name")
            commands.append(DOMAIN_CODE % domain)
            commands.append(NO_ALLOW_HASH_CODE)
            if tracking_type == TRACK_MULTIPLE_DOMAINS:
                commands.append(ALLOW_LINKER_CODE)
        return commands

    def _get_custom_var_commands(self, context):
        values = (
            context.get('google_analytics_var%s' % i) for i in range(1, 6)
        )
        params = [(i, v) for i, v in enumerate(values, 1) if v is not None]
        commands = []
        for index, var in params:
            name = var[0]
            value = var[1]
            try:
                scope = var[2]
            except IndexError:
                scope = SCOPE_PAGE
            commands.append(CUSTOM_VAR_CODE % {
                'index': index,
                'name': name,
                'value': value,
                'scope': scope,
            })
        return commands

    def _get_other_commands(self, context):
        commands = []
        if getattr(settings, 'GOOGLE_ANALYTICS_SITE_SPEED', False):
            commands.append(SITE_SPEED_CODE)

        if getattr(settings, 'GOOGLE_ANALYTICS_ANONYMIZE_IP', False):
            commands.append(ANONYMIZE_IP_CODE)

        sampleRate = getattr(settings, 'GOOGLE_ANALYTICS_SAMPLE_RATE', False)
        if sampleRate is not False:
            value = decimal.Decimal(sampleRate)
            if not 0 <= value <= 100:
                raise AnalyticalException("'GOOGLE_ANALYTICS_SAMPLE_RATE' must be >= 0 and <= 100")
            commands.append(SAMPLE_RATE_CODE % value.quantize(TWOPLACES))

        siteSpeedSampleRate = getattr(settings, 'GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE', False)
        if siteSpeedSampleRate is not False:
            value = decimal.Decimal(siteSpeedSampleRate)
            if not 0 <= value <= 100:
                raise AnalyticalException(
                    "'GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE' must be >= 0 and <= 100")
            commands.append(SITE_SPEED_SAMPLE_RATE_CODE % value.quantize(TWOPLACES))

        sessionCookieTimeout = getattr(settings, 'GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT', False)
        if sessionCookieTimeout is not False:
            value = decimal.Decimal(sessionCookieTimeout)
            if value < 0:
                raise AnalyticalException("'GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT' must be >= 0")
            commands.append(SESSION_COOKIE_TIMEOUT_CODE % value.quantize(ZEROPLACES))

        visitorCookieTimeout = getattr(settings, 'GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT', False)
        if visitorCookieTimeout is not False:
            value = decimal.Decimal(visitorCookieTimeout)
            if value < 0:
                raise AnalyticalException("'GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT' must be >= 0")
            commands.append(VISITOR_COOKIE_TIMEOUT_CODE % value.quantize(ZEROPLACES))
        return commands


def contribute_to_analytical(add_node):
    GoogleAnalyticsNode()  # ensure properly configured
    add_node('head_bottom', GoogleAnalyticsNode)



"""
Tests for the Crazy Egg template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.crazy_egg import CrazyEggNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(CRAZY_EGG_ACCOUNT_NUMBER='12345678')
class CrazyEggTagTestCase(TagTestCase):
    """
    Tests for the ``crazy_egg`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('crazy_egg', 'crazy_egg')
        self.assertTrue('/1234/5678.js' in r, r)

    def test_node(self):
        r = CrazyEggNode().render(Context())
        self.assertTrue('/1234/5678.js' in r, r)

    @override_settings(CRAZY_EGG_ACCOUNT_NUMBER=None)
    def test_no_account_number(self):
        self.assertRaises(AnalyticalException, CrazyEggNode)

    @override_settings(CRAZY_EGG_ACCOUNT_NUMBER='123abc')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, CrazyEggNode)

    def test_uservars(self):
        context = Context({'crazy_egg_var1': 'foo', 'crazy_egg_var2': 'bar'})
        r = CrazyEggNode().render(context)
        self.assertTrue("CE2.set(1, 'foo');" in r, r)
        self.assertTrue("CE2.set(2, 'bar');" in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = CrazyEggNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Crazy Egg disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the Facebook Pixel template tags.
"""
from django.http import HttpRequest
from django.template import Context, Template, TemplateSyntaxError
from django.test import override_settings

from analytical.templatetags.analytical import _load_template_nodes
from analytical.templatetags.facebook_pixel import FacebookPixelHeadNode, FacebookPixelBodyNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


expected_head_html = """\
<script>
  !function(f,b,e,v,n,t,s)
  {if(f.fbq)return;n=f.fbq=function(){n.callMethod?
  n.callMethod.apply(n,arguments):n.queue.push(arguments)};
  if(!f._fbq)f._fbq=n;n.push=n;n.loaded=!0;n.version='2.0';
  n.queue=[];t=b.createElement(e);t.async=!0;
  t.src=v;s=b.getElementsByTagName(e)[0];
  s.parentNode.insertBefore(t,s)}(window, document,'script',
  'https://connect.facebook.net/en_US/fbevents.js');
  fbq('init', '1234567890');
  fbq('track', 'PageView');
</script>
"""


expected_body_html = """\
<noscript><img height="1" width="1" style="display:none"
  src="https://www.facebook.com/tr?id=1234567890&ev=PageView&noscript=1"
/></noscript>
"""


@override_settings(FACEBOOK_PIXEL_ID='1234567890')
class FacebookPixelTagTestCase(TagTestCase):

    maxDiff = None

    def test_head_tag(self):
        html = self.render_tag('facebook_pixel', 'facebook_pixel_head')
        self.assertEqual(expected_head_html, html)

    def test_head_node(self):
        html = FacebookPixelHeadNode().render(Context({}))
        self.assertEqual(expected_head_html, html)

    def test_body_tag(self):
        html = self.render_tag('facebook_pixel', 'facebook_pixel_body')
        self.assertEqual(expected_body_html, html)

    def test_body_node(self):
        html = FacebookPixelBodyNode().render(Context({}))
        self.assertEqual(expected_body_html, html)

    def test_tags_take_no_args(self):
        self.assertRaisesRegexp(
            TemplateSyntaxError,
            r"^'facebook_pixel_head' takes no arguments$",
            lambda: (Template('{% load facebook_pixel %}{% facebook_pixel_head "arg" %}')
                     .render(Context({}))),
        )
        self.assertRaisesRegexp(
            TemplateSyntaxError,
            r"^'facebook_pixel_body' takes no arguments$",
            lambda: (Template('{% load facebook_pixel %}{% facebook_pixel_body "arg" %}')
                     .render(Context({}))),
        )

    @override_settings(FACEBOOK_PIXEL_ID=None)
    def test_no_id(self):
        expected_pattern = r'^FACEBOOK_PIXEL_ID setting is not set$'
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, FacebookPixelHeadNode)
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, FacebookPixelBodyNode)

    @override_settings(FACEBOOK_PIXEL_ID='invalid')
    def test_invalid_id(self):
        expected_pattern = (
            r"^FACEBOOK_PIXEL_ID setting: must be \(a string containing\) a number: 'invalid'$")
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, FacebookPixelHeadNode)
        self.assertRaisesRegexp(AnalyticalException, expected_pattern, FacebookPixelBodyNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': request})

        def _disabled(html):
            return '\n'.join([
                '<!-- Facebook Pixel disabled on internal IP address',
                html,
                '-->',
            ])

        head_html = FacebookPixelHeadNode().render(context)
        self.assertEqual(_disabled(expected_head_html), head_html)

        body_html = FacebookPixelBodyNode().render(context)
        self.assertEqual(_disabled(expected_body_html), body_html)

    def test_contribute_to_analytical(self):
        """
        `facebook_pixel.contribute_to_analytical` registers the head and body nodes.
        """
        template_nodes = _load_template_nodes()
        self.assertEqual({
            'head_top': [],
            'head_bottom': [FacebookPixelHeadNode],
            'body_top': [],
            'body_bottom': [FacebookPixelBodyNode],
        }, template_nodes)



"""
Tests for the generic template tags and filters.
"""

from django.template import Context, Template

from analytical.templatetags import analytical
from analytical.tests.utils import TagTestCase


class AnalyticsTagTestCase(TagTestCase):
    """
    Tests for the ``analytical`` template tags.
    """

    def setUp(self):
        super(AnalyticsTagTestCase, self).setUp()
        self._tag_modules = analytical.TAG_MODULES
        analytical.TAG_MODULES = ['analytical.tests.dummy']
        analytical.template_nodes = analytical._load_template_nodes()

    def tearDown(self):
        analytical.TAG_MODULES = self._tag_modules
        analytical.template_nodes = analytical._load_template_nodes()
        super(AnalyticsTagTestCase, self).tearDown()

    def render_location_tag(self, location, vars=None):
        if vars is None:
            vars = {}
        t = Template("{%% load analytical %%}{%% analytical_%s %%}" % location)
        return t.render(Context(vars))

    def test_location_tags(self):
        for l in ['head_top', 'head_bottom', 'body_top', 'body_bottom']:
            r = self.render_location_tag(l)
            self.assertTrue('dummy_%s' % l in r, r)



"""
Tests for the Olark template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.olark import OlarkNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(OLARK_SITE_ID='1234-567-89-0123')
class OlarkTestCase(TagTestCase):
    """
    Tests for the ``olark`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('olark', 'olark')
        self.assertTrue("olark.identify('1234-567-89-0123');" in r, r)

    def test_node(self):
        r = OlarkNode().render(Context())
        self.assertTrue("olark.identify('1234-567-89-0123');" in r, r)

    @override_settings(OLARK_SITE_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, OlarkNode)

    @override_settings(OLARK_SITE_ID='1234-567-8901234')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, OlarkNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = OlarkNode().render(Context({
            'user': User(username='test', first_name='Test', last_name='User'),
        }))
        self.assertTrue("olark('api.chat.updateVisitorNickname', "
                        "{snippet: 'Test User (test)'});" in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = OlarkNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse("olark('api.chat.updateVisitorNickname', " in r, r)

    def test_nickname(self):
        r = OlarkNode().render(Context({'olark_nickname': 'testnick'}))
        self.assertTrue("olark('api.chat.updateVisitorNickname', "
                        "{snippet: 'testnick'});" in r, r)

    def test_status_string(self):
        r = OlarkNode().render(Context({'olark_status': 'teststatus'}))
        self.assertTrue("olark('api.chat.updateVisitorStatus', "
                        '{snippet: "teststatus"});' in r, r)

    def test_status_string_list(self):
        r = OlarkNode().render(Context({
            'olark_status': ['teststatus1', 'teststatus2'],
        }))
        self.assertTrue("olark('api.chat.updateVisitorStatus', "
                        '{snippet: ["teststatus1", "teststatus2"]});' in r, r)

    def test_messages(self):
        messages = [
            "welcome_title",
            "chatting_title",
            "unavailable_title",
            "busy_title",
            "away_message",
            "loading_title",
            "welcome_message",
            "busy_message",
            "chat_input_text",
            "name_input_text",
            "email_input_text",
            "offline_note_message",
            "send_button_text",
            "offline_note_thankyou_text",
            "offline_note_error_text",
            "offline_note_sending_text",
            "operator_is_typing_text",
            "operator_has_stopped_typing_text",
            "introduction_error_text",
            "introduction_messages",
            "introduction_submit_button_text",
        ]
        vars = dict(('olark_%s' % m, m) for m in messages)
        r = OlarkNode().render(Context(vars))
        for m in messages:
            self.assertTrue("olark.configure('locale.%s', \"%s\");" % (m, m) in r, r)



"""
Mixpanel template tags and filters.
"""

from __future__ import absolute_import

import json
import re

from django.template import Library, Node, TemplateSyntaxError
from django.utils.safestring import mark_safe

from analytical.utils import is_internal_ip, disable_html, get_identity, \
        get_required_setting


MIXPANEL_API_TOKEN_RE = re.compile(r'^[0-9a-f]{32}$')
TRACKING_CODE = """
    <script type="text/javascript">(function(e,b){if(!b.__SV){var a,f,i,g;window.mixpanel=b;a=e.createElement("script");a.type="text/javascript";a.async=!0;a.src=("https:"===e.location.protocol?"https:":"http:")+'//cdn.mxpnl.com/libs/mixpanel-2.2.min.js';f=e.getElementsByTagName("script")[0];f.parentNode.insertBefore(a,f);b._i=[];b.init=function(a,e,d){function f(b,h){var a=h.split(".");2==a.length&&(b=b[a[0]],h=a[1]);b[h]=function(){b.push([h].concat(Array.prototype.slice.call(arguments,0)))}}var c=b;"undefined"!==
typeof d?c=b[d]=[]:d="mixpanel";c.people=c.people||[];c.toString=function(b){var a="mixpanel";"mixpanel"!==d&&(a+="."+d);b||(a+=" (stub)");return a};c.people.toString=function(){return c.toString(1)+".people (stub)"};i="disable track track_pageview track_links track_forms register register_once alias unregister identify name_tag set_config people.set people.increment people.append people.track_charge people.clear_charges people.delete_user".split(" ");for(g=0;g<i.length;g++)f(c,i[g]);b._i.push([a,
e,d])};b.__SV=1.2}})(document,window.mixpanel||[]);
    mixpanel.init('%(token)s');
    %(commands)s
    </script>
"""  # noqa
IDENTIFY_CODE = "mixpanel.identify('%s');"
IDENTIFY_PROPERTIES = "mixpanel.people.set(%s);"
EVENT_CODE = "mixpanel.track('%(name)s', %(properties)s);"
EVENT_CONTEXT_KEY = 'mixpanel_event'

register = Library()


@register.tag
def mixpanel(parser, token):
    """
    Mixpanel tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Mixpanel token in the ``MIXPANEL_API_TOKEN`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return MixpanelNode()


class MixpanelNode(Node):
    def __init__(self):
        self._token = get_required_setting(
                'MIXPANEL_API_TOKEN', MIXPANEL_API_TOKEN_RE,
                "must be a string containing a 32-digit hexadecimal number")

    def render(self, context):
        commands = []
        identity = get_identity(context, 'mixpanel')
        if identity is not None:
            if isinstance(identity, dict):
                commands.append(IDENTIFY_CODE % identity.get('id', identity.get('username')))
                commands.append(IDENTIFY_PROPERTIES % json.dumps(identity, sort_keys=True))
            else:
                commands.append(IDENTIFY_CODE % identity)
        try:
            name, properties = context[EVENT_CONTEXT_KEY]
            commands.append(EVENT_CODE % {
                'name': name,
                'properties': json.dumps(properties, sort_keys=True),
            })
        except KeyError:
            pass
        html = TRACKING_CODE % {
            'token': self._token,
            'commands': " ".join(commands),
        }
        if is_internal_ip(context, 'MIXPANEL'):
            html = disable_html(html, 'Mixpanel')
        return mark_safe(html)


def contribute_to_analytical(add_node):
    MixpanelNode()  # ensure properly configured
    add_node('head_bottom', MixpanelNode)



"""
Rating@Mail.ru template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, \
        get_required_setting


COUNTER_ID_RE = re.compile(r'^\d{7}$')
COUNTER_CODE = """
    <script type="text/javascript">
    var _tmr = window._tmr || (window._tmr = []);
    _tmr.push({id: "%(counter_id)s", type: "pageView", start: (new Date()).getTime()});
    (function (d, w, id) {
      if (d.getElementById(id)) return;
      var ts = d.createElement("script"); ts.type = "text/javascript"; ts.async = true; ts.id = id;
      ts.src = (d.location.protocol == "https:" ? "https:" : "http:") + "//top-fwz1.mail.ru/js/code.js";
      var f = function () {var s = d.getElementsByTagName("script")[0]; s.parentNode.insertBefore(ts, s);};
      if (w.opera == "[object Opera]") { d.addEventListener("DOMContentLoaded", f, false); } else { f(); }
    })(document, window, "topmailru-code");
    </script>
    <noscript><div style="position:absolute;left:-10000px;">
    <img src="//top-fwz1.mail.ru/counter?id=%(counter_id)s;js=na" style="border:0;" height="1" width="1" alt="Rating@Mail.ru" />
    </div></noscript>
"""  # noqa


register = Library()


@register.tag
def rating_mailru(parser, token):
    """
    Rating@Mail.ru counter template tag.

    Renders Javascript code to track page visits. You must supply
    your website counter ID (as a string) in the
    ``RATING_MAILRU_COUNTER_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return RatingMailruNode()


class RatingMailruNode(Node):
    def __init__(self):
        self.counter_id = get_required_setting(
            'RATING_MAILRU_COUNTER_ID', COUNTER_ID_RE,
            "must be (a string containing) a number'")

    def render(self, context):
        html = COUNTER_CODE % {
            'counter_id': self.counter_id,
        }
        if is_internal_ip(context, 'RATING_MAILRU_METRICA'):
            html = disable_html(html, 'Rating@Mail.ru')
        return html


def contribute_to_analytical(add_node):
    RatingMailruNode()  # ensure properly configured
    add_node('head_bottom', RatingMailruNode)



"""
Tests for the Clicky template tags and filters.
"""

import re

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.clicky import ClickyNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(CLICKY_SITE_ID='12345678')
class ClickyTagTestCase(TagTestCase):
    """
    Tests for the ``clicky`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('clicky', 'clicky')
        self.assertTrue('clicky_site_ids.push(12345678);' in r, r)
        self.assertTrue('src="//in.getclicky.com/12345678ns.gif"' in r, r)

    def test_node(self):
        r = ClickyNode().render(Context({}))
        self.assertTrue('clicky_site_ids.push(12345678);' in r, r)
        self.assertTrue('src="//in.getclicky.com/12345678ns.gif"' in r, r)

    @override_settings(CLICKY_SITE_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, ClickyNode)

    @override_settings(CLICKY_SITE_ID='123abc')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, ClickyNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = ClickyNode().render(Context({'user': User(username='test')}))
        self.assertTrue('var clicky_custom = {"session": {"username": "test"}};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = ClickyNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse('var clicky_custom = {"session": {"username":' in r, r)

    def test_custom(self):
        r = ClickyNode().render(Context({
            'clicky_var1': 'val1',
            'clicky_var2': 'val2',
        }))
        self.assertTrue(
            re.search(r'var clicky_custom = {.*"var1": "val1", "var2": "val2".*};', r),
            r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = ClickyNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Clicky disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the Performable template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.performable import PerformableNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(PERFORMABLE_API_KEY='123ABC')
class PerformableTagTestCase(TagTestCase):
    """
    Tests for the ``performable`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('performable', 'performable')
        self.assertTrue('/performable/pax/123ABC.js' in r, r)

    def test_node(self):
        r = PerformableNode().render(Context())
        self.assertTrue('/performable/pax/123ABC.js' in r, r)

    @override_settings(PERFORMABLE_API_KEY=None)
    def test_no_api_key(self):
        self.assertRaises(AnalyticalException, PerformableNode)

    @override_settings(PERFORMABLE_API_KEY='123 ABC')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, PerformableNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = PerformableNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Performable disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = PerformableNode().render(Context({'user': User(username='test')}))
        self.assertTrue('_paq.push(["identify", {identity: "test"}]);' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = PerformableNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse('_paq.push(["identify", ' in r, r)


class PerformableEmbedTagTestCase(TagTestCase):
    """
    Tests for the ``performable_embed`` template tag.
    """

    def test_tag(self):
        domain = 'example.com'
        page = 'test'
        tag = self.render_tag(
            'performable', 'performable_embed "%s" "%s"' % (domain, page)
        )
        self.assertIn(
            "$f.initialize({'host': 'example.com', 'page': 'test'});", tag
        )



"""
Clickmap template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting


CLICKMAP_TRACKER_ID_RE = re.compile(r'^\w+$')
TRACKING_CODE = """
    <script type="text/javascript">
    var clickmapConfig = {tracker: '%(tracker_id)s', version:'2'};
    window.clickmapAsyncInit = function(){ __clickmap.init(clickmapConfig); };
    (function() { var _cmf = document.createElement('script'); _cmf.async = true;
    _cmf.src = document.location.protocol + '//www.clickmap.ch/tracker.js?t=';
    _cmf.src += clickmapConfig.tracker; _cmf.id += 'clickmap_tracker';
    _cmf.src += '&v='+clickmapConfig.version+'&now='+(new Date().getTime());
    if (document.getElementById('clickmap_tracker')==null) {
    document.getElementsByTagName('head')[0].appendChild(_cmf); }}());
    </script>
"""

register = Library()


@register.tag
def clickmap(parser, token):
    """
    Clickmap tracker template tag.

    Renders Javascript code to track page visits.  You must supply
    your clickmap tracker ID (as a string) in the ``CLICKMAP_TRACKER_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return ClickmapNode()


class ClickmapNode(Node):
    def __init__(self):
        self.tracker_id = get_required_setting('CLICKMAP_TRACKER_ID',
                                               CLICKMAP_TRACKER_ID_RE,
                                               "must be an alphanumeric string")

    def render(self, context):
        html = TRACKING_CODE % {'tracker_id': self.tracker_id}
        if is_internal_ip(context, 'CLICKMAP'):
            html = disable_html(html, 'Clickmap')
        return html


def contribute_to_analytical(add_node):
    ClickmapNode()  # ensure properly configured
    add_node('body_bottom', ClickmapNode)



"""
KISSmetrics template tags.
"""

from __future__ import absolute_import

import json
import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_identity, \
        get_required_setting


API_KEY_RE = re.compile(r'^[0-9a-f]{40}$')
TRACKING_CODE = """
    <script type="text/javascript">
      var _kmq = _kmq || [];
      %(commands)s
      function _kms(u){
        setTimeout(function(){
          var s = document.createElement('script');
          s.type = 'text/javascript';
          s.async = true;
          s.src = u;
          var f = document.getElementsByTagName('script')[0];
          f.parentNode.insertBefore(s, f);
        }, 1);
      }
      _kms('//i.kissmetrics.com/i.js');
      _kms('//doug1izaerwt3.cloudfront.net/%(api_key)s.1.js');
    </script>
"""
IDENTIFY_CODE = "_kmq.push(['identify', '%s']);"
EVENT_CODE = "_kmq.push(['record', '%(name)s', %(properties)s]);"
PROPERTY_CODE = "_kmq.push(['set', %(properties)s]);"
ALIAS_CODE = "_kmq.push(['alias', '%s', '%s']);"

EVENT_CONTEXT_KEY = 'kiss_metrics_event'
PROPERTY_CONTEXT_KEY = 'kiss_metrics_properties'
ALIAS_CONTEXT_KEY = 'kiss_metrics_alias'

register = Library()


@register.tag
def kiss_metrics(parser, token):
    """
    KISSinsights tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your KISSmetrics API key in the ``KISS_METRICS_API_KEY``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return KissMetricsNode()


class KissMetricsNode(Node):
    def __init__(self):
        self.api_key = get_required_setting(
            'KISS_METRICS_API_KEY', API_KEY_RE,
            "must be a string containing a 40-digit hexadecimal number")

    def render(self, context):
        commands = []
        identity = get_identity(context, 'kiss_metrics')
        if identity is not None:
            commands.append(IDENTIFY_CODE % identity)
        try:
            properties = context[ALIAS_CONTEXT_KEY]
            key, value = properties.popitem()
            commands.append(ALIAS_CODE % (key, value))
        except KeyError:
            pass
        try:
            name, properties = context[EVENT_CONTEXT_KEY]
            commands.append(EVENT_CODE % {
                'name': name,
                'properties': json.dumps(properties, sort_keys=True),
            })
        except KeyError:
            pass
        try:
            properties = context[PROPERTY_CONTEXT_KEY]
            commands.append(PROPERTY_CODE % {
                'properties': json.dumps(properties, sort_keys=True),
            })
        except KeyError:
            pass
        html = TRACKING_CODE % {
            'api_key': self.api_key,
            'commands': " ".join(commands),
        }
        if is_internal_ip(context, 'KISS_METRICS'):
            html = disable_html(html, 'KISSmetrics')
        return html


def contribute_to_analytical(add_node):
    KissMetricsNode()  # ensure properly configured
    add_node('head_top', KissMetricsNode)



"""
HubSpot template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting


PORTAL_ID_RE = re.compile(r'^\d+$')
TRACKING_CODE = """
    <!-- Start of Async HubSpot Analytics Code -->
      <script type="text/javascript">
        (function(d,s,i,r) {
          if (d.getElementById(i)){return;}
          var n=d.createElement(s),e=d.getElementsByTagName(s)[0];
          n.id=i;n.src='//js.hs-analytics.net/analytics/'+(Math.ceil(new Date()/r)*r)+'/%(portal_id)s.js';
          e.parentNode.insertBefore(n, e);
        })(document,"script","hs-analytics",300000);
      </script>
    <!-- End of Async HubSpot Analytics Code -->
"""  # noqa

register = Library()


@register.tag
def hubspot(parser, token):
    """
    HubSpot tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your portal ID (as a string) in the ``HUBSPOT_PORTAL_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return HubSpotNode()


class HubSpotNode(Node):
    def __init__(self):
        self.portal_id = get_required_setting('HUBSPOT_PORTAL_ID', PORTAL_ID_RE,
                                              "must be a (string containing a) number")

    def render(self, context):
        html = TRACKING_CODE % {'portal_id': self.portal_id}
        if is_internal_ip(context, 'HUBSPOT'):
            html = disable_html(html, 'HubSpot')
        return html


def contribute_to_analytical(add_node):
    HubSpotNode()  # ensure properly configured
    add_node('body_bottom', HubSpotNode)



"""
Hotjar template tags and filters.
"""
from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_required_setting, is_internal_ip, disable_html


HOTJAR_TRACKING_CODE = """\
<script>
    (function(h,o,t,j,a,r){
        h.hj=h.hj||function(){(h.hj.q=h.hj.q||[]).push(arguments)};
        h._hjSettings={hjid:%(HOTJAR_SITE_ID)s,hjsv:6};
        a=o.getElementsByTagName('head')[0];
        r=o.createElement('script');r.async=1;
        r.src=t+h._hjSettings.hjid+j+h._hjSettings.hjsv;
        a.appendChild(r);
    })(window,document,'https://static.hotjar.com/c/hotjar-','.js?sv=');
</script>
"""


register = Library()


def _validate_no_args(token):
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])


@register.tag
def hotjar(parser, token):
    """
    Hotjar template tag.
    """
    _validate_no_args(token)
    return HotjarNode()


class HotjarNode(Node):

    def __init__(self):
        self.site_id = get_required_setting(
            'HOTJAR_SITE_ID',
            re.compile(r'^\d+$'),
            "must be (a string containing) a number",
        )

    def render(self, context):
        html = HOTJAR_TRACKING_CODE % {'HOTJAR_SITE_ID': self.site_id}
        if is_internal_ip(context, 'HOTJAR'):
            return disable_html(html, 'Hotjar')
        else:
            return html


def contribute_to_analytical(add_node):
    # ensure properly configured
    HotjarNode()
    add_node('head_bottom', HotjarNode)



"""
Tests for the Google Analytics template tags and filters, using the new analytics.js library.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.google_analytics_js import GoogleAnalyticsJsNode, \
    TRACK_SINGLE_DOMAIN, TRACK_MULTIPLE_DOMAINS, TRACK_MULTIPLE_SUBDOMAINS
from analytical.tests.utils import TestCase, TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GOOGLE_ANALYTICS_JS_PROPERTY_ID='UA-123456-7',
                   GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_SINGLE_DOMAIN)
class GoogleAnalyticsTagTestCase(TagTestCase):
    """
    Tests for the ``google_analytics_js`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('google_analytics_js', 'google_analytics_js')
        self.assertTrue("""(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
})(window,document,'script','https://www.google-analytics.com/analytics.js','ga');""" in r, r)
        self.assertTrue("ga('create', 'UA-123456-7', 'auto', {});" in r, r)
        self.assertTrue("ga('send', 'pageview');" in r, r)

    def test_node(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("""(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
})(window,document,'script','https://www.google-analytics.com/analytics.js','ga');""" in r, r)
        self.assertTrue("ga('create', 'UA-123456-7', 'auto', {});" in r, r)
        self.assertTrue("ga('send', 'pageview');" in r, r)

    @override_settings(GOOGLE_ANALYTICS_JS_PROPERTY_ID=None)
    def test_no_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode)

    @override_settings(GOOGLE_ANALYTICS_JS_PROPERTY_ID='wrong')
    def test_wrong_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode)

    @override_settings(GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_SUBDOMAINS,
                       GOOGLE_ANALYTICS_DOMAIN='example.com')
    def test_track_multiple_subdomains(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue(
            """ga('create', 'UA-123456-7', 'auto', {"legacyCookieDomain": "example.com"}""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_DOMAINS,
                       GOOGLE_ANALYTICS_DOMAIN='example.com')
    def test_track_multiple_domains(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("ga('create', 'UA-123456-7', 'auto', {" in r, r)
        self.assertTrue('"legacyCookieDomain": "example.com"' in r, r)
        self.assertTrue('"allowLinker\": true' in r, r)

    def test_custom_vars(self):
        context = Context({
            'google_analytics_var1': ('test1', 'foo'),
            'google_analytics_var2': ('test2', 'bar'),
            'google_analytics_var4': ('test4', 1),
            'google_analytics_var5': ('test5', 2.2),
        })
        r = GoogleAnalyticsJsNode().render(context)
        self.assertTrue("ga('set', 'test1', 'foo');" in r, r)
        self.assertTrue("ga('set', 'test2', 'bar');" in r, r)
        self.assertTrue("ga('set', 'test4', 1);" in r, r)
        self.assertTrue("ga('set', 'test5', 2.2);" in r, r)

    def test_display_advertising(self):
        with override_settings(GOOGLE_ANALYTICS_DISPLAY_ADVERTISING=True):
            r = GoogleAnalyticsJsNode().render(Context())
            self.assertTrue("""ga('create', 'UA-123456-7', 'auto', {});
ga('require', 'displayfeatures');
ga('send', 'pageview');""" in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GoogleAnalyticsJsNode().render(context)
        self.assertTrue(r.startswith(
            '<!-- Google Analytics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    @override_settings(GOOGLE_ANALYTICS_ANONYMIZE_IP=True)
    def test_anonymize_ip(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("ga('set', 'anonymizeIp', true);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_ANONYMIZE_IP=False)
    def test_anonymize_ip_not_present(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertFalse("ga('set', 'anonymizeIp', true);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=0.0)
    def test_set_sample_rate_min(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("""ga('create', 'UA-123456-7', 'auto', {"sampleRate": 0});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE='100.00')
    def test_set_sample_rate_max(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("""ga('create', 'UA-123456-7', 'auto', {"sampleRate": 100});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=-1)
    def test_exception_whenset_sample_rate_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=101)
    def test_exception_when_set_sample_rate_too_large(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=0.0)
    def test_set_site_speed_sample_rate_min(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue(
            """ga('create', 'UA-123456-7', 'auto', {"siteSpeedSampleRate": 0});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE='100.00')
    def test_set_site_speed_sample_rate_max(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue(
            """ga('create', 'UA-123456-7', 'auto', {"siteSpeedSampleRate": 100});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=-1)
    def test_exception_whenset_site_speed_sample_rate_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=101)
    def test_exception_when_set_site_speed_sample_rate_too_large(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_COOKIE_EXPIRATION=0)
    def test_set_cookie_expiration_min(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue("""ga('create', 'UA-123456-7', 'auto', {"cookieExpires": 0});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_COOKIE_EXPIRATION='10000')
    def test_set_cookie_expiration_as_string(self):
        r = GoogleAnalyticsJsNode().render(Context())
        self.assertTrue(
            """ga('create', 'UA-123456-7', 'auto', {"cookieExpires": 10000});""" in r, r)

    @override_settings(GOOGLE_ANALYTICS_COOKIE_EXPIRATION=-1)
    def test_exception_when_set_cookie_expiration_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)


@override_settings(GOOGLE_ANALYTICS_JS_PROPERTY_ID='UA-123456-7',
                   GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_DOMAINS,
                   GOOGLE_ANALYTICS_DOMAIN=None,
                   ANALYTICAL_DOMAIN=None)
class NoDomainTestCase(TestCase):
    def test_exception_without_domain(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsJsNode().render, context)



"""
Tests for the Gauges template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.gauges import GaugesNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GAUGES_SITE_ID='1234567890abcdef0123456789')
class GaugesTagTestCase(TagTestCase):
    """
    Tests for the ``gauges`` template tag.
    """

    def test_tag(self):
        self.assertEqual("""
    <script type="text/javascript">
      var _gauges = _gauges || [];
      (function() {
        var t   = document.createElement('script');
        t.type  = 'text/javascript';
        t.async = true;
        t.id    = 'gauges-tracker';
        t.setAttribute('data-site-id', '1234567890abcdef0123456789');
        t.src = '//secure.gaug.es/track.js';
        var s = document.getElementsByTagName('script')[0];
        s.parentNode.insertBefore(t, s);
      })();
    </script>
""", self.render_tag('gauges', 'gauges'))

    def test_node(self):
        self.assertEqual(
                """
    <script type="text/javascript">
      var _gauges = _gauges || [];
      (function() {
        var t   = document.createElement('script');
        t.type  = 'text/javascript';
        t.async = true;
        t.id    = 'gauges-tracker';
        t.setAttribute('data-site-id', '1234567890abcdef0123456789');
        t.src = '//secure.gaug.es/track.js';
        var s = document.getElementsByTagName('script')[0];
        s.parentNode.insertBefore(t, s);
      })();
    </script>
""", GaugesNode().render(Context()))

    @override_settings(GAUGES_SITE_ID=None)
    def test_no_account_number(self):
        self.assertRaises(AnalyticalException, GaugesNode)

    @override_settings(GAUGES_SITE_ID='123abQ')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, GaugesNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GaugesNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Gauges disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the GoSquared template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.gosquared import GoSquaredNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GOSQUARED_SITE_TOKEN='ABC-123456-D')
class GoSquaredTagTestCase(TagTestCase):
    """
    Tests for the ``gosquared`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('gosquared', 'gosquared')
        self.assertTrue('GoSquared.acct = "ABC-123456-D";' in r, r)

    def test_node(self):
        r = GoSquaredNode().render(Context({}))
        self.assertTrue('GoSquared.acct = "ABC-123456-D";' in r, r)

    @override_settings(GOSQUARED_SITE_TOKEN=None)
    def test_no_token(self):
        self.assertRaises(AnalyticalException, GoSquaredNode)

    @override_settings(GOSQUARED_SITE_TOKEN='this is not a token')
    def test_wrong_token(self):
        self.assertRaises(AnalyticalException, GoSquaredNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_auto_identify(self):
        r = GoSquaredNode().render(Context({
            'user': User(username='test', first_name='Test', last_name='User'),
        }))
        self.assertTrue('GoSquared.UserName = "Test User";' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_manual_identify(self):
        r = GoSquaredNode().render(Context({
            'user': User(username='test', first_name='Test', last_name='User'),
            'gosquared_identity': 'test_identity',
        }))
        self.assertTrue('GoSquared.UserName = "test_identity";' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = GoSquaredNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse('GoSquared.UserName = ' in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GoSquaredNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- GoSquared disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the Woopra template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.woopra import WoopraNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(WOOPRA_DOMAIN='example.com')
class WoopraTagTestCase(TagTestCase):
    """
    Tests for the ``woopra`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('woopra', 'woopra')
        self.assertTrue('var woo_settings = {"domain": "example.com"};' in r, r)

    def test_node(self):
        r = WoopraNode().render(Context({}))
        self.assertTrue('var woo_settings = {"domain": "example.com"};' in r, r)

    @override_settings(WOOPRA_DOMAIN=None)
    def test_no_domain(self):
        self.assertRaises(AnalyticalException, WoopraNode)

    @override_settings(WOOPRA_DOMAIN='this is not a domain')
    def test_wrong_domain(self):
        self.assertRaises(AnalyticalException, WoopraNode)

    @override_settings(WOOPRA_IDLE_TIMEOUT=1234)
    def test_idle_timeout(self):
        r = WoopraNode().render(Context({}))
        self.assertTrue('var woo_settings = '
                        '{"domain": "example.com", "idle_timeout": "1234"};' in r, r)

    def test_custom(self):
        r = WoopraNode().render(Context({
            'woopra_var1': 'val1',
            'woopra_var2': 'val2',
        }))
        self.assertTrue('var woo_visitor = {"var1": "val1", "var2": "val2"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_name_and_email(self):
        r = WoopraNode().render(Context({
            'user': User(username='test',
                         first_name='Firstname',
                         last_name='Lastname',
                         email="test@example.com"),
        }))
        self.assertTrue('var woo_visitor = '
                        '{"email": "test@example.com", "name": "Firstname Lastname"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_username_no_email(self):
        r = WoopraNode().render(Context({'user': User(username='test')}))
        self.assertTrue('var woo_visitor = {"name": "test"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_no_identify_when_explicit_name(self):
        r = WoopraNode().render(Context({
            'woopra_name': 'explicit',
            'user': User(username='implicit'),
        }))
        self.assertTrue('var woo_visitor = {"name": "explicit"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_no_identify_when_explicit_email(self):
        r = WoopraNode().render(Context({
            'woopra_email': 'explicit',
            'user': User(username='implicit'),
        }))
        self.assertTrue('var woo_visitor = {"email": "explicit"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = WoopraNode().render(Context({'user': AnonymousUser()}))
        self.assertTrue('var woo_visitor = {};' in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = WoopraNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Woopra disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the Clickmap template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.clickmap import ClickmapNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(CLICKMAP_TRACKER_ID='12345ABC')
class ClickmapTagTestCase(TagTestCase):
    """
    Tests for the ``clickmap`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('clickmap', 'clickmap')
        self.assertTrue("tracker: '12345ABC', version:'2'};" in r, r)

    def test_node(self):
        r = ClickmapNode().render(Context({}))
        self.assertTrue("tracker: '12345ABC', version:'2'};" in r, r)

    @override_settings(CLICKMAP_TRACKER_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, ClickmapNode)

    @override_settings(CLICKMAP_TRACKER_ID='ab#c')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, ClickmapNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = ClickmapNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Clickmap disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



def setup(app):
    app.add_crossref_type(
        directivename="setting",
        rolename="setting",
        indextemplate="pair: %s; setting",
    )
    app.add_crossref_type(
        directivename="templatetag",
        rolename="ttag",
        indextemplate="pair: %s; template tag"
    )
    app.add_crossref_type(
        directivename="templatefilter",
        rolename="tfilter",
        indextemplate="pair: %s; template filter"
    )
    app.add_crossref_type(
        directivename="fieldlookup",
        rolename="lookup",
        indextemplate="pair: %s; field lookup type",
    )



"""
Analytics service integration for Django projects
"""

__author__ = "Joost Cassee"
__email__ = "joost@cassee.net"
__version__ = "2.6.0"
__copyright__ = "Copyright (C) 2011-2020 Joost Cassee and contributors"
__license__ = "MIT"



"""
Tests for the KISSinsights template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.kiss_insights import KissInsightsNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(KISS_INSIGHTS_ACCOUNT_NUMBER='12345', KISS_INSIGHTS_SITE_CODE='abc')
class KissInsightsTagTestCase(TagTestCase):
    """
    Tests for the ``kiss_insights`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('kiss_insights', 'kiss_insights')
        self.assertTrue("//s3.amazonaws.com/ki.js/12345/abc.js" in r, r)

    def test_node(self):
        r = KissInsightsNode().render(Context())
        self.assertTrue("//s3.amazonaws.com/ki.js/12345/abc.js" in r, r)

    @override_settings(KISS_INSIGHTS_ACCOUNT_NUMBER=None)
    def test_no_account_number(self):
        self.assertRaises(AnalyticalException, KissInsightsNode)

    @override_settings(KISS_INSIGHTS_SITE_CODE=None)
    def test_no_site_code(self):
        self.assertRaises(AnalyticalException, KissInsightsNode)

    @override_settings(KISS_INSIGHTS_ACCOUNT_NUMBER='abcde')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, KissInsightsNode)

    @override_settings(KISS_INSIGHTS_SITE_CODE='abc def')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, KissInsightsNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = KissInsightsNode().render(Context({'user': User(username='test')}))
        self.assertTrue("_kiq.push(['identify', 'test']);" in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = KissInsightsNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse("_kiq.push(['identify', " in r, r)

    def test_show_survey(self):
        r = KissInsightsNode().render(Context({'kiss_insights_show_survey': 1234}))
        self.assertTrue("_kiq.push(['showSurvey', 1234]);" in r, r)



"""
KISSinsights template tags.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_identity, get_required_setting


ACCOUNT_NUMBER_RE = re.compile(r'^\d+$')
SITE_CODE_RE = re.compile(r'^[\w]+$')
SETUP_CODE = """
    <script type="text/javascript">var _kiq = _kiq || []; %(commands)s</script>
    <script type="text/javascript" src="//s3.amazonaws.com/ki.js/%(account_number)s/%(site_code)s.js" async="true"></script>
"""  # noqa
IDENTIFY_CODE = "_kiq.push(['identify', '%s']);"
SHOW_SURVEY_CODE = "_kiq.push(['showSurvey', %s]);"
SHOW_SURVEY_CONTEXT_KEY = 'kiss_insights_show_survey'


register = Library()


@register.tag
def kiss_insights(parser, token):
    """
    KISSinsights set-up template tag.

    Renders Javascript code to set-up surveys.  You must supply
    your account number and site code in the
    ``KISS_INSIGHTS_ACCOUNT_NUMBER`` and ``KISS_INSIGHTS_SITE_CODE``
    settings.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return KissInsightsNode()


class KissInsightsNode(Node):
    def __init__(self):
        self.account_number = get_required_setting(
            'KISS_INSIGHTS_ACCOUNT_NUMBER', ACCOUNT_NUMBER_RE,
            "must be (a string containing) a number")
        self.site_code = get_required_setting(
            'KISS_INSIGHTS_SITE_CODE', SITE_CODE_RE,
            "must be a string containing three characters")

    def render(self, context):
        commands = []
        identity = get_identity(context, 'kiss_insights')
        if identity is not None:
            commands.append(IDENTIFY_CODE % identity)
        try:
            commands.append(SHOW_SURVEY_CODE % context[SHOW_SURVEY_CONTEXT_KEY])
        except KeyError:
            pass
        html = SETUP_CODE % {
            'account_number': self.account_number,
            'site_code': self.site_code,
            'commands': " ".join(commands),
        }
        return html


def contribute_to_analytical(add_node):
    KissInsightsNode()  # ensure properly configured
    add_node('body_top', KissInsightsNode)



"""
Spring Metrics template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_identity, is_internal_ip, disable_html, \
        get_required_setting


TRACKING_ID_RE = re.compile(r'^[0-9a-f]+$')
TRACKING_CODE = """
    <script type='text/javascript'>
     var _springMetq = _springMetq || [];
     _springMetq.push(['id', '%(tracking_id)s']);
     (
      function(){
       var s = document.createElement('script');
       s.type = 'text/javascript';
       s.async = true;
       s.src = ('https:' == document.location.protocol ? 'https://d3rmnwi2tssrfx.cloudfront.net/a.js' : 'http://static.springmetrics.com/a.js');
       var x = document.getElementsByTagName('script')[0];
       x.parentNode.insertBefore(s, x);
      }
     )();
     %(custom_commands)s
    </script>
"""  # noqa

register = Library()


@register.tag
def spring_metrics(parser, token):
    """
    Spring Metrics tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Spring Metrics Tracking ID in the
    ``SPRING_METRICS_TRACKING_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return SpringMetricsNode()


class SpringMetricsNode(Node):
    def __init__(self):
        self.tracking_id = get_required_setting(
                'SPRING_METRICS_TRACKING_ID',
                TRACKING_ID_RE, "must be a hexadecimal string")

    def render(self, context):
        custom = {}
        for dict_ in context:
            for var, val in dict_.items():
                if var.startswith('spring_metrics_'):
                    custom[var[15:]] = val
        if 'email' not in custom:
            identity = get_identity(context, 'spring_metrics',
                                    lambda u: u.email)
            if identity is not None:
                custom['email'] = identity

        html = TRACKING_CODE % {
            'tracking_id': self.tracking_id,
            'custom_commands': self._generate_custom_javascript(custom),
        }
        if is_internal_ip(context, 'SPRING_METRICS'):
            html = disable_html(html, 'Spring Metrics')
        return html

    def _generate_custom_javascript(self, params):
        commands = []
        convert = params.pop('convert', None)
        if convert is not None:
            commands.append("_springMetq.push(['convert', '%s'])" % convert)
        commands.extend("_springMetq.push(['setdata', {'%s': '%s'}]);"
                        % (var, val) for var, val in params.items())
        return " ".join(commands)


def contribute_to_analytical(add_node):
    SpringMetricsNode()  # ensure properly configured
    add_node('head_bottom', SpringMetricsNode)



"""
Testing utilities.
"""

from __future__ import with_statement

from django.template import Template, Context, RequestContext
from django.test.testcases import TestCase


def run_tests():
    """
    Use the Django test runner to run the tests.

    Sets the return code to the number of failed tests.
    """
    import sys
    import django
    try:
        django.setup()
    except AttributeError:
        pass
    try:
        from django.test.runner import DiscoverRunner as TestRunner
    except ImportError:
        from django.test.simple import DjangoTestSuiteRunner as TestRunner
    runner = TestRunner()
    sys.exit(runner.run_tests(["analytical"]))


class TagTestCase(TestCase):
    """
    Tests for a template tag.

    Adds support methods for testing template tags.
    """

    def render_tag(self, library, tag, vars=None, request=None):
        if vars is None:
            vars = {}
        t = Template("{%% load %s %%}{%% %s %%}" % (library, tag))
        if request is not None:
            context = RequestContext(request, vars)
        else:
            context = Context(vars)
        return t.render(context)

    def render_template(self, template, vars=None, request=None):
        if vars is None:
            vars = {}
        t = Template(template)
        if request is not None:
            context = RequestContext(request, vars)
        else:
            context = Context(vars)
        return t.render(context)



"""
Yandex.Metrica template tags and filters.
"""

from __future__ import absolute_import

import json
import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, \
        get_required_setting


COUNTER_ID_RE = re.compile(r'^\d{8}$')
COUNTER_CODE = """
    <script type="text/javascript">
        (function (d, w, c) {
            (w[c] = w[c] || []).push(function() {
                try {
                    w.yaCounter%(counter_id)s = new Ya.Metrika(%(options)s);
                } catch(e) { }
            });

            var n = d.getElementsByTagName("script")[0],
                s = d.createElement("script"),
                f = function () { n.parentNode.insertBefore(s, n); };
            s.type = "text/javascript";
            s.async = true;
            s.src = "https://mc.yandex.ru/metrika/watch.js";

            if (w.opera == "[object Opera]") {
                d.addEventListener("DOMContentLoaded", f, false);
            } else { f(); }
        })(document, window, "yandex_metrika_callbacks");
    </script>
    <noscript><div><img src="https://mc.yandex.ru/watch/%(counter_id)s" style="position:absolute; left:-9999px;" alt="" /></div></noscript>
"""  # noqa


register = Library()


@register.tag
def yandex_metrica(parser, token):
    """
    Yandex.Metrica counter template tag.

    Renders Javascript code to track page visits. You must supply
    your website counter ID (as a string) in the
    ``YANDEX_METRICA_COUNTER_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return YandexMetricaNode()


class YandexMetricaNode(Node):
    def __init__(self):
        self.counter_id = get_required_setting(
                'YANDEX_METRICA_COUNTER_ID', COUNTER_ID_RE,
                "must be (a string containing) a number'")

    def render(self, context):
        options = {
            'id': int(self.counter_id),
            'clickmap': True,
            'trackLinks': True,
            'accurateTrackBounce': True
        }
        if getattr(settings, 'YANDEX_METRICA_WEBVISOR', False):
            options['webvisor'] = True
        if getattr(settings, 'YANDEX_METRICA_TRACKHASH', False):
            options['trackHash'] = True
        if getattr(settings, 'YANDEX_METRICA_NOINDEX', False):
            options['ut'] = 'noindex'
        if getattr(settings, 'YANDEX_METRICA_ECOMMERCE', False):
            options['ecommerce'] = 'dataLayer'
        html = COUNTER_CODE % {
            'counter_id': self.counter_id,
            'options': json.dumps(options),
        }
        if is_internal_ip(context, 'YANDEX_METRICA'):
            html = disable_html(html, 'Yandex.Metrica')
        return html


def contribute_to_analytical(add_node):
    YandexMetricaNode()  # ensure properly configured
    add_node('head_bottom', YandexMetricaNode)



"""
Woopra template tags and filters.
"""

from __future__ import absolute_import

import json
import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import (
    disable_html,
    get_identity,
    get_required_setting,
    get_user_from_context,
    get_user_is_authenticated,
    is_internal_ip,
)

DOMAIN_RE = re.compile(r'^\S+$')
TRACKING_CODE = """
     <script type="text/javascript">
      var woo_settings = %(settings)s;
      var woo_visitor = %(visitor)s;
      !function(){var a,b,c,d=window,e=document,f=arguments,g="script",h=["config","track","trackForm","trackClick","identify","visit","push","call"],i=function(){var a,b=this,c=function(a){b[a]=function(){return b._e.push([a].concat(Array.prototype.slice.call(arguments,0))),b}};for(b._e=[],a=0;a<h.length;a++)c(h[a])};for(d.__woo=d.__woo||{},a=0;a<f.length;a++)d.__woo[f[a]]=d[f[a]]=d[f[a]]||new i;b=e.createElement(g),b.async=1,b.src="//static.woopra.com/js/w.js",c=e.getElementsByTagName(g)[0],c.parentNode.insertBefore(b,c)}("woopra");
      woopra.config(woo_settings);
      woopra.identify(woo_visitor);
      woopra.track();
    </script>
"""  # noqa

register = Library()


@register.tag
def woopra(parser, token):
    """
    Woopra tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Woopra domain in the ``WOOPRA_DOMAIN`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return WoopraNode()


class WoopraNode(Node):
    def __init__(self):
        self.domain = get_required_setting(
            'WOOPRA_DOMAIN', DOMAIN_RE,
            "must be a domain name")

    def render(self, context):
        settings = self._get_settings(context)
        visitor = self._get_visitor(context)

        html = TRACKING_CODE % {
            'settings': json.dumps(settings, sort_keys=True),
            'visitor': json.dumps(visitor, sort_keys=True),
        }
        if is_internal_ip(context, 'WOOPRA'):
            html = disable_html(html, 'Woopra')
        return html

    def _get_settings(self, context):
        variables = {'domain': self.domain}
        try:
            variables['idle_timeout'] = str(settings.WOOPRA_IDLE_TIMEOUT)
        except AttributeError:
            pass
        return variables

    def _get_visitor(self, context):
        params = {}
        for dict_ in context:
            for var, val in dict_.items():
                if var.startswith('woopra_'):
                    params[var[7:]] = val
        if 'name' not in params and 'email' not in params:
            user = get_user_from_context(context)
            if user is not None and get_user_is_authenticated(user):
                params['name'] = get_identity(
                    context, 'woopra', self._identify, user)
                if user.email:
                    params['email'] = user.email
        return params

    def _identify(self, user):
        name = user.get_full_name()
        if not name:
            name = user.username
        return name


def contribute_to_analytical(add_node):
    WoopraNode()  # ensure properly configured
    add_node('head_bottom', WoopraNode)



"""
Google Analytics template tags and filters, using the new analytics.js library.
"""

from __future__ import absolute_import

import decimal
import re
from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import (
    AnalyticalException,
    disable_html,
    get_domain,
    get_required_setting,
    is_internal_ip,
)

TRACK_SINGLE_DOMAIN = 1
TRACK_MULTIPLE_SUBDOMAINS = 2
TRACK_MULTIPLE_DOMAINS = 3

PROPERTY_ID_RE = re.compile(r'^UA-\d+-\d+$')
SETUP_CODE = """
<script>
(function(i,s,o,g,r,a,m){{i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){{
(i[r].q=i[r].q||[]).push(arguments)}},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
}})(window,document,'script','https://www.google-analytics.com/analytics.js','ga');

ga('create', '{property_id}', 'auto', {create_fields});
{commands}ga('send', 'pageview');
</script>
"""
REQUIRE_DISPLAY_FEATURES = "ga('require', 'displayfeatures');\n"
CUSTOM_VAR_CODE = "ga('set', '{name}', {value});\n"
ANONYMIZE_IP_CODE = "ga('set', 'anonymizeIp', true);\n"

register = Library()


@register.tag
def google_analytics_js(parser, token):
    """
    Google Analytics tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your website property ID (as a string) in the
    ``GOOGLE_ANALYTICS_JS_PROPERTY_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return GoogleAnalyticsJsNode()


class GoogleAnalyticsJsNode(Node):
    def __init__(self):
        self.property_id = get_required_setting(
            'GOOGLE_ANALYTICS_JS_PROPERTY_ID', PROPERTY_ID_RE,
            "must be a string looking like 'UA-XXXXXX-Y'")

    def render(self, context):
        import json
        create_fields = self._get_domain_fields(context)
        create_fields.update(self._get_other_create_fields(context))
        commands = self._get_custom_var_commands(context)
        commands.extend(self._get_other_commands(context))
        display_features = getattr(settings, 'GOOGLE_ANALYTICS_DISPLAY_ADVERTISING', False)
        if display_features:
            commands.insert(0, REQUIRE_DISPLAY_FEATURES)

        html = SETUP_CODE.format(
            property_id=self.property_id,
            create_fields=json.dumps(create_fields),
            commands="".join(commands),
        )
        if is_internal_ip(context, 'GOOGLE_ANALYTICS'):
            html = disable_html(html, 'Google Analytics')
        return html

    def _get_domain_fields(self, context):
        domain_fields = {}
        tracking_type = getattr(settings, 'GOOGLE_ANALYTICS_TRACKING_STYLE', TRACK_SINGLE_DOMAIN)
        if tracking_type == TRACK_SINGLE_DOMAIN:
            pass
        else:
            domain = get_domain(context, 'google_analytics')
            if domain is None:
                raise AnalyticalException(
                    "tracking multiple domains with Google Analytics requires a domain name")
            domain_fields['legacyCookieDomain'] = domain
            if tracking_type == TRACK_MULTIPLE_DOMAINS:
                domain_fields['allowLinker'] = True
        return domain_fields

    def _get_other_create_fields(self, context):
        other_fields = {}

        site_speed_sample_rate = getattr(settings, 'GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE', False)
        if site_speed_sample_rate is not False:
            value = int(decimal.Decimal(site_speed_sample_rate))
            if not 0 <= value <= 100:
                raise AnalyticalException(
                    "'GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE' must be >= 0 and <= 100")
            other_fields['siteSpeedSampleRate'] = value

        sample_rate = getattr(settings, 'GOOGLE_ANALYTICS_SAMPLE_RATE', False)
        if sample_rate is not False:
            value = int(decimal.Decimal(sample_rate))
            if not 0 <= value <= 100:
                raise AnalyticalException("'GOOGLE_ANALYTICS_SAMPLE_RATE' must be >= 0 and <= 100")
            other_fields['sampleRate'] = value

        cookie_expires = getattr(settings, 'GOOGLE_ANALYTICS_COOKIE_EXPIRATION', False)
        if cookie_expires is not False:
            value = int(decimal.Decimal(cookie_expires))
            if value < 0:
                raise AnalyticalException("'GOOGLE_ANALYTICS_COOKIE_EXPIRATION' must be >= 0")
            other_fields['cookieExpires'] = value

        return other_fields

    def _get_custom_var_commands(self, context):
        values = (
            context.get('google_analytics_var%s' % i) for i in range(1, 6)
        )
        params = [(i, v) for i, v in enumerate(values, 1) if v is not None]
        commands = []
        for _, var in params:
            name = var[0]
            value = var[1]
            try:
                float(value)
            except ValueError:
                value = "'{}'".format(value)
            commands.append(CUSTOM_VAR_CODE.format(
                name=name,
                value=value,
            ))
        return commands

    def _get_other_commands(self, context):
        commands = []

        if getattr(settings, 'GOOGLE_ANALYTICS_ANONYMIZE_IP', False):
            commands.append(ANONYMIZE_IP_CODE)

        return commands


def contribute_to_analytical(add_node):
    GoogleAnalyticsJsNode()  # ensure properly configured
    add_node('head_bottom', GoogleAnalyticsJsNode)



"""
intercom.io template tags and filters.
"""

from __future__ import absolute_import

import hashlib
import hmac
import json
import sys
import re
import time

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import disable_html, get_required_setting, \
        is_internal_ip, get_user_from_context, get_identity, \
        get_user_is_authenticated

APP_ID_RE = re.compile(r'[\da-z]+$')
TRACKING_CODE = """
<script id="IntercomSettingsScriptTag">
  window.intercomSettings = %(settings_json)s;
</script>
<script>(function(){var w=window;var ic=w.Intercom;if(typeof ic==="function"){ic('reattach_activator');ic('update',intercomSettings);}else{var d=document;var i=function(){i.c(arguments)};i.q=[];i.c=function(args){i.q.push(args)};w.Intercom=i;function l(){var s=d.createElement('script');s.type='text/javascript';s.async=true;s.src='https://static.intercomcdn.com/intercom.v1.js';var x=d.getElementsByTagName('script')[0];x.parentNode.insertBefore(s,x);}if(w.attachEvent){w.attachEvent('onload',l);}else{w.addEventListener('load',l,false);}}})()</script>
"""  # noqa

register = Library()


def _timestamp(when):
    """
    Python 2 compatibility for `datetime.timestamp()`.
    """
    return (time.mktime(when.timetuple()) if sys.version_info < (3,) else
            when.timestamp())


def _hashable_bytes(data):
    """
    Coerce strings to hashable bytes.
    """
    if isinstance(data, bytes):
        return data
    elif isinstance(data, str):
        return data.encode('ascii')  # Fail on anything non-ASCII.
    else:
        raise TypeError(data)


def intercom_user_hash(data):
    """
    Return a SHA-256 HMAC `user_hash` as expected by Intercom, if configured.

    Return None if the `INTERCOM_HMAC_SECRET_KEY` setting is not configured.
    """
    if getattr(settings, 'INTERCOM_HMAC_SECRET_KEY', None):
        return hmac.new(
            key=_hashable_bytes(settings.INTERCOM_HMAC_SECRET_KEY),
            msg=_hashable_bytes(data),
            digestmod=hashlib.sha256,
        ).hexdigest()
    else:
        return None


@register.tag
def intercom(parser, token):
    """
    Intercom.io template tag.

    Renders Javascript code to intercom.io testing.  You must supply
    your APP ID account number in the ``INTERCOM_APP_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return IntercomNode()


class IntercomNode(Node):
    def __init__(self):
        self.app_id = get_required_setting(
                'INTERCOM_APP_ID', APP_ID_RE,
                "must be a string looking like 'XXXXXXX'")

    def _identify(self, user):
        name = user.get_full_name()
        if not name:
            name = user.username
        return name

    def _get_custom_attrs(self, context):
        params = {}
        for dict_ in context:
            for var, val in dict_.items():
                if var.startswith('intercom_'):
                    params[var[9:]] = val

        user = get_user_from_context(context)
        if user is not None and get_user_is_authenticated(user):
            if 'name' not in params:
                params['name'] = get_identity(
                        context, 'intercom', self._identify, user)
            if 'email' not in params and user.email:
                params['email'] = user.email

            params.setdefault('user_id', user.pk)

            params['created_at'] = int(_timestamp(user.date_joined))
        else:
            params['created_at'] = None

        # Generate a user_hash HMAC to verify the user's identity, if configured.
        # (If both user_id and email are present, the user_id field takes precedence.)
        # See:
        # https://www.intercom.com/help/configure-intercom-for-your-product-or-site/staying-secure/enable-identity-verification-on-your-web-product
        user_hash_data = params.get('user_id', params.get('email'))
        if user_hash_data:
            user_hash = intercom_user_hash(str(user_hash_data))
            if user_hash is not None:
                params.setdefault('user_hash', user_hash)

        return params

    def render(self, context):
        params = self._get_custom_attrs(context)
        params["app_id"] = self.app_id
        html = TRACKING_CODE % {
            "settings_json": json.dumps(params, sort_keys=True)
        }

        if is_internal_ip(context, 'INTERCOM'):
            html = disable_html(html, 'Intercom')
        return html


def contribute_to_analytical(add_node):
    IntercomNode()
    add_node('body_bottom', IntercomNode)



"""
Optimizely template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting


ACCOUNT_NUMBER_RE = re.compile(r'^\d+$')
SETUP_CODE = """<script src="//cdn.optimizely.com/js/%(account_number)s.js"></script>"""


register = Library()


@register.tag
def optimizely(parser, token):
    """
    Optimizely template tag.

    Renders Javascript code to set-up A/B testing.  You must supply
    your Optimizely account number in the ``OPTIMIZELY_ACCOUNT_NUMBER``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return OptimizelyNode()


class OptimizelyNode(Node):
    def __init__(self):
        self.account_number = get_required_setting(
                'OPTIMIZELY_ACCOUNT_NUMBER', ACCOUNT_NUMBER_RE,
                "must be a string looking like 'XXXXXXX'")

    def render(self, context):
        html = SETUP_CODE % {'account_number': self.account_number}
        if is_internal_ip(context, 'OPTIMIZELY'):
            html = disable_html(html, 'Optimizely')
        return html


def contribute_to_analytical(add_node):
    OptimizelyNode()  # ensure properly configured
    add_node('head_top', OptimizelyNode)



"""
Tests for the HubSpot template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.hubspot import HubSpotNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(HUBSPOT_PORTAL_ID='1234')
class HubSpotTagTestCase(TagTestCase):
    """
    Tests for the ``hubspot`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('hubspot', 'hubspot')
        self.assertTrue("n.id=i;n.src='//js.hs-analytics.net/analytics/'"
                        "+(Math.ceil(new Date()/r)*r)+'/1234.js';" in r, r)

    def test_node(self):
        r = HubSpotNode().render(Context())
        self.assertTrue("n.id=i;n.src='//js.hs-analytics.net/analytics/'"
                        "+(Math.ceil(new Date()/r)*r)+'/1234.js';" in r, r)

    @override_settings(HUBSPOT_PORTAL_ID=None)
    def test_no_portal_id(self):
        self.assertRaises(AnalyticalException, HubSpotNode)

    @override_settings(HUBSPOT_PORTAL_ID='wrong')
    def test_wrong_portal_id(self):
        self.assertRaises(AnalyticalException, HubSpotNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = HubSpotNode().render(context)
        self.assertTrue(r.startswith('<!-- HubSpot disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Analytical template tags and filters.
"""

from __future__ import absolute_import

import logging

from django import template
from django.template import Node, TemplateSyntaxError
from importlib import import_module

from analytical.utils import AnalyticalException


TAG_LOCATIONS = ['head_top', 'head_bottom', 'body_top', 'body_bottom']
TAG_POSITIONS = ['first', None, 'last']
TAG_MODULES = [
    'analytical.chartbeat',
    'analytical.clickmap',
    'analytical.clicky',
    'analytical.crazy_egg',
    'analytical.facebook_pixel',
    'analytical.gauges',
    'analytical.google_analytics',
    'analytical.google_analytics_js',
    'analytical.google_analytics_gtag',
    'analytical.gosquared',
    'analytical.hotjar',
    'analytical.hubspot',
    'analytical.intercom',
    'analytical.kiss_insights',
    'analytical.kiss_metrics',
    'analytical.matomo',
    'analytical.mixpanel',
    'analytical.olark',
    'analytical.optimizely',
    'analytical.performable',
    'analytical.piwik',
    'analytical.rating_mailru',
    'analytical.snapengage',
    'analytical.spring_metrics',
    'analytical.uservoice',
    'analytical.woopra',
    'analytical.yandex_metrica',
]

logger = logging.getLogger(__name__)
register = template.Library()


def _location_tag(location):
    def analytical_tag(parser, token):
        bits = token.split_contents()
        if len(bits) > 1:
            raise TemplateSyntaxError("'%s' tag takes no arguments" % bits[0])
        return AnalyticalNode(location)

    return analytical_tag


for loc in TAG_LOCATIONS:
    register.tag('analytical_%s' % loc, _location_tag(loc))


class AnalyticalNode(Node):
    def __init__(self, location):
        self.nodes = [node_cls() for node_cls in template_nodes[location]]

    def render(self, context):
        return "".join([node.render(context) for node in self.nodes])


def _load_template_nodes():
    template_nodes = dict((l, dict((p, []) for p in TAG_POSITIONS))
                          for l in TAG_LOCATIONS)

    def add_node_cls(location, node, position=None):
        template_nodes[location][position].append(node)

    for path in TAG_MODULES:
        module = _import_tag_module(path)
        try:
            module.contribute_to_analytical(add_node_cls)
        except AnalyticalException as e:
            logger.debug("not loading tags from '%s': %s", path, e)
    for location in TAG_LOCATIONS:
        template_nodes[location] = sum((template_nodes[location][p]
                                        for p in TAG_POSITIONS), [])
    return template_nodes


def _import_tag_module(path):
    app_name, lib_name = path.rsplit('.', 1)
    return import_module("%s.templatetags.%s" % (app_name, lib_name))


template_nodes = _load_template_nodes()



"""
Tests for the Google Analytics template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.google_analytics import GoogleAnalyticsNode, \
        TRACK_SINGLE_DOMAIN, TRACK_MULTIPLE_DOMAINS, TRACK_MULTIPLE_SUBDOMAINS,\
        SCOPE_VISITOR, SCOPE_SESSION, SCOPE_PAGE
from analytical.tests.utils import TestCase, TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GOOGLE_ANALYTICS_PROPERTY_ID='UA-123456-7',
                   GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_SINGLE_DOMAIN)
class GoogleAnalyticsTagTestCase(TagTestCase):
    """
    Tests for the ``google_analytics`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('google_analytics', 'google_analytics')
        self.assertTrue("_gaq.push(['_setAccount', 'UA-123456-7']);" in r, r)
        self.assertTrue("_gaq.push(['_trackPageview']);" in r, r)

    def test_node(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setAccount', 'UA-123456-7']);" in r, r)
        self.assertTrue("_gaq.push(['_trackPageview']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_PROPERTY_ID=None)
    def test_no_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode)

    @override_settings(GOOGLE_ANALYTICS_PROPERTY_ID='wrong')
    def test_wrong_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode)

    @override_settings(GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_SUBDOMAINS,
                       GOOGLE_ANALYTICS_DOMAIN='example.com')
    def test_track_multiple_subdomains(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setDomainName', 'example.com']);" in r, r)
        self.assertTrue("_gaq.push(['_setAllowHash', false]);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_DOMAINS,
                       GOOGLE_ANALYTICS_DOMAIN='example.com')
    def test_track_multiple_domains(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setDomainName', 'example.com']);" in r, r)
        self.assertTrue("_gaq.push(['_setAllowHash', false]);" in r, r)
        self.assertTrue("_gaq.push(['_setAllowLinker', true]);" in r, r)

    def test_custom_vars(self):
        context = Context({
            'google_analytics_var1': ('test1', 'foo'),
            'google_analytics_var2': ('test2', 'bar', SCOPE_VISITOR),
            'google_analytics_var4': ('test4', 'baz', SCOPE_SESSION),
            'google_analytics_var5': ('test5', 'qux', SCOPE_PAGE),
        })
        r = GoogleAnalyticsNode().render(context)
        self.assertTrue("_gaq.push(['_setCustomVar', 1, 'test1', 'foo', 3]);" in r, r)
        self.assertTrue("_gaq.push(['_setCustomVar', 2, 'test2', 'bar', 1]);" in r, r)
        self.assertTrue("_gaq.push(['_setCustomVar', 4, 'test4', 'baz', 2]);" in r, r)
        self.assertTrue("_gaq.push(['_setCustomVar', 5, 'test5', 'qux', 3]);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED=True)
    def test_track_page_load_time(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_trackPageLoadTime']);" in r, r)

    def test_display_advertising(self):
        with override_settings(GOOGLE_ANALYTICS_DISPLAY_ADVERTISING=False):
            r = GoogleAnalyticsNode().render(Context())
            self.assertTrue("google-analytics.com/ga.js" in r, r)
        with override_settings(GOOGLE_ANALYTICS_DISPLAY_ADVERTISING=True):
            r = GoogleAnalyticsNode().render(Context())
            self.assertTrue("stats.g.doubleclick.net/dc.js" in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GoogleAnalyticsNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Google Analytics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    @override_settings(GOOGLE_ANALYTICS_ANONYMIZE_IP=True)
    def test_anonymize_ip(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_gat._anonymizeIp']);" in r, r)
        self.assertTrue(r.index('_gat._anonymizeIp') < r.index('_trackPageview'), r)

    @override_settings(GOOGLE_ANALYTICS_ANONYMIZE_IP=False)
    def test_anonymize_ip_not_present(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertFalse("_gaq.push(['_gat._anonymizeIp']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=0.0)
    def test_set_sample_rate_min(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSampleRate', '0.00']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE='100.00')
    def test_set_sample_rate_max(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSampleRate', '100.00']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=-1)
    def test_exception_whenset_sample_rate_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SAMPLE_RATE=101)
    def test_exception_when_set_sample_rate_too_large(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=0.0)
    def test_set_site_speed_sample_rate_min(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSiteSpeedSampleRate', '0.00']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE='100.00')
    def test_set_site_speed_sample_rate_max(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSiteSpeedSampleRate', '100.00']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=-1)
    def test_exception_whenset_site_speed_sample_rate_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE=101)
    def test_exception_when_set_site_speed_sample_rate_too_large(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT=0)
    def test_set_session_cookie_timeout_min(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSessionCookieTimeout', '0']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT='10000')
    def test_set_session_cookie_timeout_as_string(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setSessionCookieTimeout', '10000']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT=-1)
    def test_exception_when_set_session_cookie_timeout_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)

    @override_settings(GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT=0)
    def test_set_visitor_cookie_timeout_min(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setVisitorCookieTimeout', '0']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT='10000')
    def test_set_visitor_cookie_timeout_as_string(self):
        r = GoogleAnalyticsNode().render(Context())
        self.assertTrue("_gaq.push(['_setVisitorCookieTimeout', '10000']);" in r, r)

    @override_settings(GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT=-1)
    def test_exception_when_set_visitor_cookie_timeout_too_small(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)


@override_settings(GOOGLE_ANALYTICS_PROPERTY_ID='UA-123456-7',
                   GOOGLE_ANALYTICS_TRACKING_STYLE=TRACK_MULTIPLE_DOMAINS,
                   GOOGLE_ANALYTICS_DOMAIN=None,
                   ANALYTICAL_DOMAIN=None)
class NoDomainTestCase(TestCase):
    def test_exception_without_domain(self):
        context = Context()
        self.assertRaises(AnalyticalException, GoogleAnalyticsNode().render, context)



"""
Tests for the UserVoice tags and filters.
"""

from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.uservoice import UserVoiceNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(USERVOICE_WIDGET_KEY='abcdefghijklmnopqrst')
class UserVoiceTagTestCase(TagTestCase):
    """
    Tests for the ``uservoice`` template tag.
    """

    def assertIn(self, element, container):
        try:
            super(TagTestCase, self).assertIn(element, container)
        except AttributeError:
            self.assertTrue(element in container)

    def test_node(self):
        r = UserVoiceNode().render(Context())
        self.assertIn("widget.uservoice.com/abcdefghijklmnopqrst.js", r)

    def test_tag(self):
        r = self.render_tag('uservoice', 'uservoice')
        self.assertIn("widget.uservoice.com/abcdefghijklmnopqrst.js", r)

    @override_settings(USERVOICE_WIDGET_KEY=None)
    def test_no_key(self):
        self.assertRaises(AnalyticalException, UserVoiceNode)

    @override_settings(USERVOICE_WIDGET_KEY='abcdefgh ijklmnopqrst')
    def test_invalid_key(self):
        self.assertRaises(AnalyticalException, UserVoiceNode)

    @override_settings(USERVOICE_WIDGET_KEY='')
    def test_empty_key(self):
        self.assertRaises(AnalyticalException, UserVoiceNode)

    def test_overridden_key(self):
        vars = {'uservoice_widget_key': 'defghijklmnopqrstuvw'}
        r = UserVoiceNode().render(Context(vars))
        self.assertIn("widget.uservoice.com/defghijklmnopqrstuvw.js", r)

    @override_settings(USERVOICE_WIDGET_OPTIONS={'key1': 'val1'})
    def test_options(self):
        r = UserVoiceNode().render(Context())
        self.assertIn("""UserVoice.push(['set', {"key1": "val1"}]);""", r)

    @override_settings(USERVOICE_WIDGET_OPTIONS={'key1': 'val1'})
    def test_override_options(self):
        data = {'uservoice_widget_options': {'key1': 'val2'}}
        r = UserVoiceNode().render(Context(data))
        self.assertIn("""UserVoice.push(['set', {"key1": "val2"}]);""", r)

    def test_auto_trigger_default(self):
        r = UserVoiceNode().render(Context())
        self.assertTrue("UserVoice.push(['addTrigger', {}]);" in r, r)

    @override_settings(USERVOICE_ADD_TRIGGER=False)
    def test_auto_trigger(self):
        r = UserVoiceNode().render(Context())
        self.assertFalse("UserVoice.push(['addTrigger', {}]);" in r, r)

    @override_settings(USERVOICE_ADD_TRIGGER=False)
    def test_auto_trigger_custom_win(self):
        r = UserVoiceNode().render(Context({'uservoice_add_trigger': True}))
        self.assertTrue("UserVoice.push(['addTrigger', {}]);" in r, r)



"""
Olark template tags.
"""

from __future__ import absolute_import

import json
import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_identity, get_required_setting


SITE_ID_RE = re.compile(r'^\d+-\d+-\d+-\d+$')
SETUP_CODE = """
    <script type='text/javascript'>
      /*{literal}<![CDATA[*/ window.olark||(function(k){var g=window,j=document,a=g.location.protocol=="https:"?"https:":"http:",i=k.name,b="load",h="addEventListener";(function(){g[i]=function(){(c.s=c.s||[]).push(arguments)};var c=g[i]._={},f=k.methods.length;while(f--){(function(l){g[i][l]=function(){g[i]("call",l,arguments)}})(k.methods[f])}c.l=k.loader;c.i=arguments.callee;c.p={0:+new Date};c.P=function(l){c.p[l]=new Date-c.p[0]};function e(){c.P(b);g[i](b)}g[h]?g[h](b,e,false):g.attachEvent("on"+b,e);c.P(1);var d=j.createElement("script"),m=document.getElementsByTagName("script")[0];d.type="text/javascript";d.async=true;d.src=a+"//"+c.l;m.parentNode.insertBefore(d,m);c.P(2)})()})({loader:(function(a){return "static.olark.com/jsclient/loader1.js?ts="+(a?a[1]:(+new Date))})(document.cookie.match(/olarkld=([0-9]+)/)),name:"olark",methods:["configure","extend","declare","identify"]}); olark.identify('%(site_id)s');/*]]>{/literal}*/
      %(extra_code)s
    </script>
"""  # noqa
NICKNAME_CODE = "olark('api.chat.updateVisitorNickname', {snippet: '%s'});"
NICKNAME_CONTEXT_KEY = 'olark_nickname'
FULLNAME_CODE = "olark('api.visitor.updateFullName', {{fullName: '{0}'}});"
FULLNAME_CONTEXT_KEY = 'olark_fullname'
EMAIL_CODE = "olark('api.visitor.updateEmailAddress', {{emailAddress: '{0}'}});"
EMAIL_CONTEXT_KEY = 'olark_email'
STATUS_CODE = "olark('api.chat.updateVisitorStatus', {snippet: %s});"
STATUS_CONTEXT_KEY = 'olark_status'
MESSAGE_CODE = "olark.configure('locale.%(key)s', \"%(msg)s\");"
MESSAGE_KEYS = {
    "welcome_title", "chatting_title", "unavailable_title",
    "busy_title", "away_message", "loading_title", "welcome_message",
    "busy_message", "chat_input_text", "name_input_text",
    "email_input_text", "offline_note_message", "send_button_text",
    "offline_note_thankyou_text", "offline_note_error_text",
    "offline_note_sending_text", "operator_is_typing_text",
    "operator_has_stopped_typing_text", "introduction_error_text",
    "introduction_messages", "introduction_submit_button_text",
}

register = Library()


@register.tag
def olark(parser, token):
    """
    Olark set-up template tag.

    Renders Javascript code to set-up Olark chat.  You must supply
    your site ID in the ``OLARK_SITE_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return OlarkNode()


class OlarkNode(Node):
    def __init__(self):
        self.site_id = get_required_setting(
            'OLARK_SITE_ID', SITE_ID_RE,
            "must be a string looking like 'XXXX-XXX-XX-XXXX'")

    def render(self, context):
        extra_code = []
        try:
            extra_code.append(NICKNAME_CODE % context[NICKNAME_CONTEXT_KEY])
        except KeyError:
            identity = get_identity(context, 'olark', self._get_nickname)
            if identity is not None:
                extra_code.append(NICKNAME_CODE % identity)
        try:
            extra_code.append(FULLNAME_CODE.format(context[FULLNAME_CONTEXT_KEY]))
        except KeyError:
            pass
        try:
            extra_code.append(EMAIL_CODE.format(context[EMAIL_CONTEXT_KEY]))
        except KeyError:
            pass
        try:
            extra_code.append(STATUS_CODE % json.dumps(context[STATUS_CONTEXT_KEY],
                                                       sort_keys=True))
        except KeyError:
            pass
        extra_code.extend(self._get_configuration(context))
        html = SETUP_CODE % {
            'site_id': self.site_id,
            'extra_code': " ".join(extra_code),
        }
        return html

    def _get_nickname(self, user):
        name = user.get_full_name()
        if name:
            return "%s (%s)" % (name, user.username)
        else:
            return user.username

    def _get_configuration(self, context):
        code = []
        for dict_ in context:
            for var, val in dict_.items():
                if var.startswith('olark_'):
                    key = var[6:]
                    if key in MESSAGE_KEYS:
                        code.append(MESSAGE_CODE % {'key': key, 'msg': val})
        return code


def contribute_to_analytical(add_node):
    OlarkNode()  # ensure properly configured
    add_node('body_bottom', OlarkNode)



"""
Tests for the Optimizely template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.optimizely import OptimizelyNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(OPTIMIZELY_ACCOUNT_NUMBER='1234567')
class OptimizelyTagTestCase(TagTestCase):
    """
    Tests for the ``optimizely`` template tag.
    """

    def test_tag(self):
        self.assertEqual(
                '<script src="//cdn.optimizely.com/js/1234567.js"></script>',
                self.render_tag('optimizely', 'optimizely'))

    def test_node(self):
        self.assertEqual(
                '<script src="//cdn.optimizely.com/js/1234567.js"></script>',
                OptimizelyNode().render(Context()))

    @override_settings(OPTIMIZELY_ACCOUNT_NUMBER=None)
    def test_no_account_number(self):
        self.assertRaises(AnalyticalException, OptimizelyNode)

    @override_settings(OPTIMIZELY_ACCOUNT_NUMBER='123abc')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, OptimizelyNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = OptimizelyNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Optimizely disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Performable template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError
from django.utils.safestring import mark_safe

from analytical.utils import is_internal_ip, disable_html, get_identity, \
        get_required_setting


API_KEY_RE = re.compile(r'^\w+$')
SETUP_CODE = """
    <script src="//d1nu2rn22elx8m.cloudfront.net/performable/pax/%(api_key)s.js" type="text/javascript"></script>
"""  # noqa
IDENTIFY_CODE = """
    <script type="text/javascript">
      var _paq = _paq || [];
      _paq.push(["identify", {identity: "%s"}]);
    </script>
"""
EMBED_CODE = """
    <script type="text/javascript" src="//d1nu2rn22elx8m.cloudfront.net/performable/embed/page.js"></script>
    <script type="text/javascript">
      (function() {
      var $f = new PerformableEmbed();
      $f.initialize({'host': '%(hostname)s', 'page': '%(page_id)s'});
      $f.write();
    })()
    </script>
"""  # noqa

register = Library()


@register.tag
def performable(parser, token):
    """
    Performable template tag.

    Renders Javascript code to set-up Performable tracking.  You must
    supply your Performable API key in the ``PERFORMABLE_API_KEY``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return PerformableNode()


class PerformableNode(Node):
    def __init__(self):
        self.api_key = get_required_setting(
            'PERFORMABLE_API_KEY', API_KEY_RE,
            "must be a string looking like 'XXXXX'")

    def render(self, context):
        html = SETUP_CODE % {'api_key': self.api_key}
        identity = get_identity(context, 'performable')
        if identity is not None:
            html = "%s%s" % (IDENTIFY_CODE % identity, html)
        if is_internal_ip(context, 'PERFORMABLE'):
            html = disable_html(html, 'Performable')
        return html


@register.simple_tag
def performable_embed(hostname, page_id):
    """
    Include a Performable landing page.
    """
    return mark_safe(EMBED_CODE % {
        'hostname': hostname,
        'page_id': page_id,
    })


def contribute_to_analytical(add_node):
    PerformableNode()  # ensure properly configured
    add_node('body_bottom', PerformableNode)



"""
Tests for the Mixpanel tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.mixpanel import MixpanelNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(MIXPANEL_API_TOKEN='0123456789abcdef0123456789abcdef')
class MixpanelTagTestCase(TagTestCase):
    """
    Tests for the ``mixpanel`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('mixpanel', 'mixpanel')
        self.assertIn("mixpanel.init('0123456789abcdef0123456789abcdef');", r)

    def test_node(self):
        r = MixpanelNode().render(Context())
        self.assertIn("mixpanel.init('0123456789abcdef0123456789abcdef');", r)

    @override_settings(MIXPANEL_API_TOKEN=None)
    def test_no_token(self):
        self.assertRaises(AnalyticalException, MixpanelNode)

    @override_settings(MIXPANEL_API_TOKEN='0123456789abcdef0123456789abcdef0')
    def test_token_too_long(self):
        self.assertRaises(AnalyticalException, MixpanelNode)

    @override_settings(MIXPANEL_API_TOKEN='0123456789abcdef0123456789abcde')
    def test_token_too_short(self):
        self.assertRaises(AnalyticalException, MixpanelNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = MixpanelNode().render(Context({'user': User(username='test')}))
        self.assertIn("mixpanel.identify('test');", r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = MixpanelNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse("mixpanel.register_once({distinct_id:" in r, r)

    def test_event(self):
        r = MixpanelNode().render(Context({
            'mixpanel_event': ('test_event', {'prop1': 'val1', 'prop2': 'val2'}),
        }))
        self.assertTrue("mixpanel.track('test_event', "
                        '{"prop1": "val1", "prop2": "val2"});' in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = MixpanelNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Mixpanel disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)
